# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests for google.protobuf.internal.service_reflection."""

__author__ = 'petar@google.com (Petar Petrov)'


import unittest

from google.protobuf import service_reflection
from google.protobuf import unittest_pb2


class FooUnitTest(unittest.TestCase):

  def testService(self):

    class MockRpcChannel:
      def CallMethod(self, method, controller, request, response, callback):
        self.method = method
        self.controller = controller
        self.request = request
        callback(response)

    class MockRpcController:
      def SetFailed(self, msg):
        self.failure_message = msg

    self.callback_response = None

    class MyService(unittest_pb2.TestService):
      pass

    self.callback_response = None

    def MyCallback(response):
      self.callback_response = response

    rpc_controller = MockRpcController()
    channel = MockRpcChannel()
    srvc = MyService()
    srvc.Foo(rpc_controller, unittest_pb2.FooRequest(), MyCallback)
    self.assertEqual('Method Foo not implemented.',
                     rpc_controller.failure_message)
    self.assertEqual(None, self.callback_response)

    rpc_controller.failure_message = None

    service_descriptor = unittest_pb2.TestService.GetDescriptor()
    srvc.CallMethod(service_descriptor.methods[1], rpc_controller,
                    unittest_pb2.BarRequest(), MyCallback)
    self.assertTrue(srvc.GetRequestClass(service_descriptor.methods[1]) is
                    unittest_pb2.BarRequest)
    self.assertTrue(srvc.GetResponseClass(service_descriptor.methods[1]) is
                    unittest_pb2.BarResponse)
    self.assertEqual('Method Bar not implemented.',
                     rpc_controller.failure_message)
    self.assertEqual(None, self.callback_response)

    class MyServiceImpl(unittest_pb2.TestService):
      def Foo(self, rpc_controller, request, done):
        self.foo_called = True
      def Bar(self, rpc_controller, request, done):
        self.bar_called = True

    srvc = MyServiceImpl()
    rpc_controller.failure_message = None
    srvc.Foo(rpc_controller, unittest_pb2.FooRequest(), MyCallback)
    self.assertEqual(None, rpc_controller.failure_message)
    self.assertEqual(True, srvc.foo_called)

    rpc_controller.failure_message = None
    srvc.CallMethod(service_descriptor.methods[1], rpc_controller,
                    unittest_pb2.BarRequest(), MyCallback)
    self.assertEqual(None, rpc_controller.failure_message)
    self.assertEqual(True, srvc.bar_called)

  def testServiceStub(self):

    class MockRpcChannel:
      def CallMethod(self, method, controller, request,
                     response_class, callback):
        self.method = method
        self.controller = controller
        self.request = request
        callback(response_class())

    self.callback_response = None

    def MyCallback(response):
      self.callback_response = response

    channel = MockRpcChannel()
    stub = unittest_pb2.TestService_Stub(channel)
    rpc_controller = 'controller'
    request = 'request'

    # GetDescriptor now static, still works as instance method for compatibility
    self.assertEqual(unittest_pb2.TestService_Stub.GetDescriptor(),
                     stub.GetDescriptor())

    # Invoke method.
    stub.Foo(rpc_controller, request, MyCallback)

    self.assertIsInstance(self.callback_response, unittest_pb2.FooResponse)
    self.assertEqual(request, channel.request)
    self.assertEqual(rpc_controller, channel.controller)
    self.assertEqual(stub.GetDescriptor().methods[0], channel.method)


if __name__ == '__main__':
  unittest.main()



"""Regression test for self or indirect recursive messages with pybind11."""

from google.protobuf.internal import pybind11_test_module
from google.protobuf.internal import self_recursive_from_py_pb2
from google3.testing.pybase import unittest


class RecursiveMessagePybind11Test(unittest.TestCase):

  def test_self_recursive_message_callback(self):
    called = False

    def callback(
        msg: self_recursive_from_py_pb2.ContainsSelfRecursive,
    ) -> None:
      nonlocal called
      called = True

    # Without proper handling of message factories (in pyext/message.cc New)
    # this will stack overflow
    pybind11_test_module.invoke_callback_on_message(
        callback, self_recursive_from_py_pb2.ContainsSelfRecursive()
    )
    self.assertTrue(called)

  def test_indirect_recursive_message_callback(self):
    called = False

    def callback(
        msg: self_recursive_from_py_pb2.ContainsIndirectRecursive,
    ) -> None:
      nonlocal called
      called = True

    # Without proper handling of message factories (in pyext/message.cc New)
    # this will stack overflow
    pybind11_test_module.invoke_callback_on_message(
        callback, self_recursive_from_py_pb2.ContainsIndirectRecursive()
    )
    self.assertTrue(called)


if __name__ == "__main__":
  unittest.main()



#!/usr/bin/env python3
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""A conformance test implementation for the Python protobuf library.

See conformance.proto for more information.
"""

import struct
import sys
from google.protobuf import json_format
from google.protobuf import message
from google.protobuf import text_format
from google.protobuf import test_messages_proto2_pb2
from google.protobuf import test_messages_proto3_pb2
from conformance import conformance_pb2
from conformance.test_protos import test_messages_edition2023_pb2
from conformance.test_protos import test_messages_edition_unstable_pb2
from editions.golden import test_messages_proto2_editions_pb2
from editions.golden import test_messages_proto3_editions_pb2

test_count = 0
verbose = False


class ProtocolError(Exception):
  pass


def _create_test_message(type):
  if type == "protobuf_test_messages.proto2.TestAllTypesProto2":
    return test_messages_proto2_pb2.TestAllTypesProto2()
  if type == "protobuf_test_messages.proto3.TestAllTypesProto3":
    return test_messages_proto3_pb2.TestAllTypesProto3()
  if type == "protobuf_test_messages.editions.TestAllTypesEdition2023":
    return test_messages_edition2023_pb2.TestAllTypesEdition2023()
  if type == "protobuf_test_messages.editions.proto2.TestAllTypesProto2":
    return test_messages_proto2_editions_pb2.TestAllTypesProto2()
  if type == "protobuf_test_messages.editions.proto3.TestAllTypesProto3":
    return test_messages_proto3_editions_pb2.TestAllTypesProto3()
  if (
      type
      == "protobuf_test_messages.edition_unstable.TestAllTypesEditionUnstable"
  ):
    return test_messages_edition_unstable_pb2.TestAllTypesEditionUnstable()
  return None


def do_test(request):
  response = conformance_pb2.ConformanceResponse()

  is_json = request.WhichOneof("payload") == "json_payload"
  test_message = _create_test_message(request.message_type)

  if (not is_json) and (test_message is None):
    raise ProtocolError("Protobuf request doesn't have specific payload type")

  try:
    if request.WhichOneof("payload") == "protobuf_payload":
      try:
        test_message.ParseFromString(request.protobuf_payload)
      except message.DecodeError as e:
        response.parse_error = str(e)
        return response

    elif request.WhichOneof("payload") == "json_payload":
      try:
        ignore_unknown_fields = (
            request.test_category
            == conformance_pb2.JSON_IGNORE_UNKNOWN_PARSING_TEST
        )
        json_format.Parse(
            request.json_payload, test_message, ignore_unknown_fields
        )
      except Exception as e:
        response.parse_error = str(e)
        return response

    elif request.WhichOneof("payload") == "text_payload":
      try:
        text_format.Parse(request.text_payload, test_message)
      except Exception as e:
        response.parse_error = str(e)
        return response

    else:
      raise ProtocolError("Request didn't have payload.")

    if request.requested_output_format == conformance_pb2.UNSPECIFIED:
      raise ProtocolError("Unspecified output format")

    elif request.requested_output_format == conformance_pb2.PROTOBUF:
      response.protobuf_payload = test_message.SerializeToString()

    elif request.requested_output_format == conformance_pb2.JSON:
      try:
        response.json_payload = json_format.MessageToJson(test_message)
      except Exception as e:
        response.serialize_error = str(e)
        return response

    elif request.requested_output_format == conformance_pb2.TEXT_FORMAT:
      response.text_payload = text_format.MessageToString(
          test_message, print_unknown_fields=request.print_unknown_fields
      )

  except Exception as e:
    response.runtime_error = str(e)

  return response


def do_test_io():
  length_bytes = sys.stdin.buffer.read(4)
  if len(length_bytes) == 0:
    return False  # EOF
  elif len(length_bytes) != 4:
    raise IOError("I/O error")

  length = struct.unpack("<I", length_bytes)[0]
  serialized_request = sys.stdin.buffer.read(length)
  if len(serialized_request) != length:
    raise IOError("I/O error")

  request = conformance_pb2.ConformanceRequest()
  request.ParseFromString(serialized_request)

  response = do_test(request)

  serialized_response = response.SerializeToString()
  sys.stdout.buffer.write(struct.pack("<I", len(serialized_response)))
  sys.stdout.buffer.write(serialized_response)
  sys.stdout.buffer.flush()

  if verbose:
    sys.stderr.write(
        "conformance_python: request=%s, response=%s\n"
        % (
            request.ShortDebugString().c_str(),
            response.ShortDebugString().c_str(),
        )
    )

  global test_count
  test_count += 1

  return True


while True:
  if not do_test_io():
    sys.stderr.write(
        "conformance_python: received EOF from test runner "
        + "after %s tests, exiting\n" % (test_count,)
    )
    sys.exit(0)



"""Exercises proto_api.h. Most of the code is in proto_api_test.cc."""

from google.protobuf.internal import proto_api_test_ext
from google3.testing.pybase import unittest
from google.protobuf import unittest_pb2


class ProtoApiTest(unittest.TestCase):

  def test_get_const_message(self):
    msg = unittest_pb2.TestAllTypes(optional_int32=123, optional_string='hello')
    result = proto_api_test_ext.get_const_message(msg)
    self.assertEqual(result, (123, 'hello'))

  def test_set_message_field_with_mutator(self):
    msg = unittest_pb2.TestAllTypes(optional_int32=123, optional_string='hello')
    proto_api_test_ext.set_message_field_with_mutator(msg, 456)
    # GetClearedMessageMutator clears message first, then update it.
    # On destruction, it copies content back to python message.
    self.assertEqual(456, msg.optional_int32)
    self.assertFalse(msg.HasField('optional_string'))

  def test_dynamic_message(self):
    result = proto_api_test_ext.repr_dynamic_message(789)
    self.assertEqual(result, 'optional_int32: 789\n')

  def test_dynamic_pool_message(self):
    result = proto_api_test_ext.create_dynamic_pool_message()
    self.assertEqual(result.DESCRIPTOR.full_name, 'test_package.MyMessage')
    self.assertEqual(result.my_field, 42)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
#
# Protocol Buffers - Google's data interchange format
# Copyright 2023 Google LLC.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""A tool to convert MODULE.bazel -> CMakeLists.txt.

This tool is very protobuf-specific at the moment, and should not be seen as a
generic Bazel -> CMake converter.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os
import re
import sys
import textwrap


class ExtensionFunctions(object):
  """A fake extension that we can use to get the functions we need."""

  def defaults(self, *args, **kwargs):
    pass

  def toolchain(self, *args, **kwargs):
    pass

  def parse(self, *args, **kwargs):
    pass

  def spec(self, *args, **kwargs):
    pass

  def from_specs(self, *args, **kwargs):
    pass

  def install(self, *args, **kwargs):
    pass

  def bundle_fetch(self, *args, **kwargs):
    pass

  def find(self, *args, **kwargs):
    pass

  def source_archive(self, *args, **kwargs):
    pass

  def nuget_package(self, *args, **kwargs):
    pass


def empty_func(*args, **kwargs):
  pass


class ModuleFileFunctions(object):
  """A fake MODULE file that we can exec() to get the functions we need."""

  def __init__(self, converter):
    self.converter = converter

  def module(self, *args, **kwargs):
    pass

  def bazel_dep(self, name, version, **kwargs):
    # Strip BCR-specific version suffixes (e.g. 1.17.0.bcr.2 -> 1.17.0)
    # since they don't correspond to upstream git tags.
    version = re.sub(r"\.bcr\.\d+", "", version)
    self.converter.toplevel += textwrap.dedent(
        """\
      set(%(name)s-version "%(version)s")
    """
        % {
            "name": name,
            "version": version,
        }
    )

  def register_toolchains(self, *args, **kwargs):
    pass

  def use_repo(self, *args, **kwargs):
    pass

  def use_repo_rule(self, *args, **kwargs):
    return empty_func

  def single_version_override(self, *args, **kwargs):
    pass

  def use_extension(self, *args, **kwargs):
    return ExtensionFunctions()

  def local_path_override(self, *args, **kwargs):
    pass

  def git_override(self, *args, **kwargs):
    pass

  def archive_override(self, *args, **kwargs):
    pass


class Converter(object):

  def __init__(self):
    self.toplevel = ""
    self.if_lua = ""

  def convert(self):
    return self.template % {
        "toplevel": converter.toplevel,
    }

  template = textwrap.dedent("""\
    # Auto-generated by @//cmake:make_dependencies
    #
    # This file contains lists of external dependencies based on our Bazel
    # config. It should be included from a hand-written CMake file that uses
    # them.
    #
    # Changes to this file will be overwritten based on Bazel definitions.

    if(${CMAKE_VERSION} VERSION_GREATER 3.16 OR ${CMAKE_VERSION} VERSION_EQUAL 3.16)
      include_guard()
    endif()

    %(toplevel)s

  """)


data = {}
converter = Converter()


def GetDict(obj):
  ret = {}
  for k in dir(obj):
    if not k.startswith("_"):
      ret[k] = getattr(obj, k)
  return ret


# We take the MODULE path as a command-line argument to ensure that we can find
# it regardless of how exactly Bazel was invoked.
exec(open(sys.argv[1]).read(), GetDict(ModuleFileFunctions(converter)))

with open(sys.argv[2], "w") as f:
  f.write(converter.convert())



# -*- coding: utf-8 -*-
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests proto Timestamp APIs."""

import datetime
import unittest

from google.protobuf import timestamp
from google.protobuf.internal import testing_refleaks
from google.protobuf.internal import well_known_types_test_pb2

from google.protobuf import timestamp_pb2


@testing_refleaks.TestCase
class TimestampTest(unittest.TestCase):

  def test_timestamp_integer_conversion(self):
    self.assertEqual(1, timestamp.to_nanoseconds(timestamp.from_nanoseconds(1)))
    self.assertEqual(-1, timestamp.to_seconds(timestamp.from_seconds(-1)))
    self.assertEqual(
        123, timestamp.to_milliseconds(timestamp.from_milliseconds(123))
    )
    self.assertEqual(
        321, timestamp.to_microseconds(timestamp.from_microseconds(321))
    )

  def test_timestamp_current(self):
    # It is not easy to check with current time. For test coverage only.
    self.assertNotEqual(8 * 3600, timestamp.from_current_time().seconds)

  def test_timestamp_json(self):

    def check_timestamp(ts, text):
      self.assertEqual(text, timestamp.to_json_string(ts))
      parsed_ts = timestamp.from_json_string(text)
      self.assertEqual(ts, parsed_ts)

    message = timestamp_pb2.Timestamp()
    message.seconds = 0
    message.nanos = 0
    check_timestamp(message, '1970-01-01T00:00:00Z')
    message.nanos = 10000000
    check_timestamp(message, '1970-01-01T00:00:00.010Z')
    message.nanos = 10000
    check_timestamp(message, '1970-01-01T00:00:00.000010Z')

  def test_timestamp_datetime(self):
    naive_utc_epoch = datetime.datetime(1970, 1, 1)
    message = well_known_types_test_pb2.WKTMessage()
    message.optional_timestamp = naive_utc_epoch
    self.assertEqual(0, message.optional_timestamp.seconds)  # pytype: disable=attribute-error
    self.assertEqual(0, message.optional_timestamp.nanos)  # pytype: disable=attribute-error
    self.assertEqual(
        naive_utc_epoch, timestamp.to_datetime(message.optional_timestamp)  # pytype: disable=wrong-arg-types
    )

  def test_timstamp_construction(self):
    message = well_known_types_test_pb2.WKTMessage(
        optional_timestamp=datetime.datetime.today()
    )

  def test_repeated_timestamp_construction(self):
    message = well_known_types_test_pb2.WKTMessage(
        repeated_ts=[
            datetime.datetime(2025, 1, 1),
            datetime.datetime(1970, 1, 1),
            timestamp_pb2.Timestamp(),
        ]
    )
    self.assertEqual(len(message.repeated_ts), 3)
    self.assertEqual(
        datetime.datetime(2025, 1, 1),
        timestamp.to_datetime((message.repeated_ts[0])),
    )
    self.assertEqual(
        datetime.datetime(1970, 1, 1),
        timestamp.to_datetime((message.repeated_ts[1])),
    )
    self.assertEqual(timestamp_pb2.Timestamp(), message.repeated_ts[2])

  def test_timestamp_sub_annotation(self):
    t1 = timestamp_pb2.Timestamp()
    t2 = timestamp_pb2.Timestamp()
    dt = datetime.datetime.now()
    td = datetime.timedelta(hours=0)
    msg = well_known_types_test_pb2.WKTMessage(optional_duration=td)
    # Timestamp - datetime
    self.assertEqual(t1 - dt, t2 - dt)
    # Timestamp - Timestamp
    self.assertEqual(t1 - t2, t2 - t1)
    # datetime - Timestamp
    self.assertEqual(dt - t1, dt - t2)
    # Timestamp - timedelta and Timestamp - Duration
    self.assertEqual(t1 - td, t2 - msg.optional_duration)

  def test_timestamp_add_annotation(self):
    ts = timestamp_pb2.Timestamp()
    td = datetime.timedelta(hours=0)
    msg = well_known_types_test_pb2.WKTMessage(optional_duration=td)
    # Timestamp + timedelta and timedelta + Timestamp
    self.assertEqual(ts + td, td + ts)
    # Timestamp + Duration and Duration + Timestamp
    self.assertEqual(ts + msg.optional_duration, msg.optional_duration + ts)

  def test_assign_duration_to_timestamp(self):
    message = well_known_types_test_pb2.WKTMessage()
    with self.assertRaises((TypeError)):
      message.optional_timestamp = datetime.timedelta(microseconds=123)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests proto Duration APIs."""

import datetime
import unittest

from google.protobuf import duration
from google.protobuf.internal import testing_refleaks
from google.protobuf.internal import well_known_types_test_pb2

from google.protobuf import duration_pb2


@testing_refleaks.TestCase
class DurationTest(unittest.TestCase):

  def test_duration_integer_conversion(self):
    self.assertEqual(1, duration.to_nanoseconds(duration.from_nanoseconds(1)))
    self.assertEqual(-1, duration.to_seconds(duration.from_seconds(-1)))
    self.assertEqual(
        123, duration.to_milliseconds(duration.from_milliseconds(123))
    )
    self.assertEqual(
        321, duration.to_microseconds(duration.from_microseconds(321))
    )

  def test_duration_json(self):

    def check_duration(message, text):
      self.assertEqual(text, duration.to_json_string(message))
      parsed_duration = duration.from_json_string(text)
      self.assertEqual(message, parsed_duration)

    message = duration_pb2.Duration()
    message.seconds = 0
    message.nanos = 0
    check_duration(message, '0s')
    message.nanos = 10000000
    check_duration(message, '0.010s')
    message.nanos = 10000
    check_duration(message, '0.000010s')
    message.nanos = 10
    check_duration(message, '0.000000010s')

  def test_duration_timedelta(self):
    message = duration.from_nanoseconds(1999999999)
    td = duration.to_timedelta(message)
    self.assertEqual(1, td.seconds)
    self.assertEqual(999999, td.microseconds)

    message = duration.from_microseconds(-1)
    td = duration.to_timedelta(message)
    converted_message = duration.from_timedelta(td)
    self.assertEqual(message, converted_message)

  def test_duration_construction(self):
    expected_td = datetime.timedelta(microseconds=123)
    message = well_known_types_test_pb2.WKTMessage(
        optional_duration=expected_td
    )
    self.assertEqual(expected_td, message.optional_duration.ToTimedelta())

  def test_repeated_duration_construction(self):
    td0 = datetime.timedelta(microseconds=123)
    td1 = datetime.timedelta(microseconds=456)
    dr = duration_pb2.Duration()
    message = well_known_types_test_pb2.WKTMessage(repeated_td=[td0, td1, dr])
    self.assertEqual(td0, duration.to_timedelta(message.repeated_td[0]))
    self.assertEqual(td1, duration.to_timedelta(message.repeated_td[1]))
    self.assertEqual(dr, message.repeated_td[2])

  def test_duration_sub_annotation(self):
    dt = datetime.datetime.now()
    dr = duration_pb2.Duration()
    td = datetime.timedelta(microseconds=123)
    # datetime - Duration
    self.assertEqual(dt - dr, dt - duration.to_timedelta(dr))
    # timedelta - Duration and Duration - Duration
    self.assertEqual(td - dr, duration.from_timedelta(td) - dr)
    # Duration - timedelta
    self.assertEqual(dr - td, dr - duration.from_timedelta(td))

  def test_duration_add_annotation(self):
    dt = datetime.datetime.now()
    dr = duration_pb2.Duration()
    dr2 = duration_pb2.Duration(seconds=100)
    # datetime + Duration and Duration + datetime
    self.assertEqual(dt + dr, dr + dt)
    message = well_known_types_test_pb2.WKTMessage(optional_timestamp=dt)
    # Duration + Timestamp
    self.assertEqual(dr + message.optional_timestamp, dr + dt)
    td = datetime.timedelta(microseconds=123)
    # Duration + timedelta and timedelta + Duration
    self.assertEqual(dr + td, td + dr)
    # Duration + Duration
    self.assertEqual(dr + dr2, dr2 + dr)

  def test_assign_datetime_to_duration(self):
    message = well_known_types_test_pb2.WKTMessage()
    with self.assertRaises((TypeError)):
      message.optional_duration = datetime.datetime.now()


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Unittest for nested public imports."""

import unittest

from google.protobuf import unknown_fields
from google.protobuf.internal.import_test_package import outer_pb2

from google.protobuf import unittest_custom_options_pb2
from google.protobuf import unittest_import_option_pb2


class ImportTest(unittest.TestCase):

  def testPackageInitializationImport(self):
    """Test that we can import nested import public messages."""

    msg = outer_pb2.Outer()
    self.assertEqual(58, msg.import_public_nested.value)

  def testImportOptionKnown(self):
    file_descriptor = unittest_import_option_pb2.DESCRIPTOR
    message_descriptor = unittest_import_option_pb2.TestMessage.DESCRIPTOR
    field_descriptor = message_descriptor.fields_by_name['field1']

    self.assertEqual(
        file_descriptor.GetOptions().Extensions[
            unittest_custom_options_pb2.file_opt1
        ],
        1,
    )
    self.assertEqual(
        message_descriptor.GetOptions().Extensions[
            unittest_custom_options_pb2.message_opt1
        ],
        2,
    )
    self.assertEqual(
        field_descriptor.GetOptions().Extensions[
            unittest_custom_options_pb2.field_opt1
        ],
        3,
    )

  def testImportOptionUnknown(self):
    file_descriptor = unittest_import_option_pb2.DESCRIPTOR
    message_descriptor = unittest_import_option_pb2.TestMessage.DESCRIPTOR
    field_descriptor = message_descriptor.fields_by_name['field1']

    # Options from import option that are not linked in should be in unknown
    # fields.
    unknown_fields_file = unknown_fields.UnknownFieldSet(
        file_descriptor.GetOptions()
    )
    unknown_fields_message = unknown_fields.UnknownFieldSet(
        message_descriptor.GetOptions()
    )
    unknown_fields_field = unknown_fields.UnknownFieldSet(
        field_descriptor.GetOptions()
    )

    self.assertEqual(len(unknown_fields_file), 1)
    self.assertEqual(unknown_fields_file[0].field_number, 7736975)
    self.assertEqual(unknown_fields_file[0].data, 1)

    self.assertEqual(len(unknown_fields_message), 1)
    self.assertEqual(unknown_fields_message[0].field_number, 7739037)
    self.assertEqual(unknown_fields_message[0].data, 2)

    self.assertEqual(len(unknown_fields_field), 1)
    self.assertEqual(unknown_fields_field[0].field_number, 7740937)
    self.assertEqual(unknown_fields_field[0].data, 3)


if __name__ == '__main__':
  unittest.main()



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Test for google.protobuf.json_format."""

__author__ = 'jieluo@google.com (Jie Luo)'

import json
import math
import struct
import unittest

from google.protobuf import descriptor_pool
from google.protobuf import json_format
from google.protobuf.internal import more_messages_pb2
from google.protobuf.internal import test_proto2_pb2
from google.protobuf.internal import test_proto3_optional_pb2

from google.protobuf import any_pb2
from google.protobuf import duration_pb2
from google.protobuf import field_mask_pb2
from google.protobuf import struct_pb2
from google.protobuf import timestamp_pb2
from google.protobuf import wrappers_pb2
from google.protobuf import any_test_pb2
from google.protobuf import unittest_mset_pb2
from google.protobuf import unittest_pb2
from google.protobuf.util import json_format_pb2
from google.protobuf.util import json_format_proto3_pb2


class JsonFormatBase(unittest.TestCase):

  def FillAllFields(self, message):
    message.int32_value = 20
    message.int64_value = -20
    message.uint32_value = 3120987654
    message.uint64_value = 12345678900
    message.float_value = float('-inf')
    message.double_value = 3.1415
    message.bool_value = True
    message.string_value = 'foo'
    message.bytes_value = b'bar'
    message.message_value.value = 10
    message.enum_value = json_format_proto3_pb2.BAR
    # Repeated
    message.repeated_int32_value.append(0x7FFFFFFF)
    message.repeated_int32_value.append(-2147483648)
    message.repeated_int64_value.append(9007199254740992)
    message.repeated_int64_value.append(-9007199254740992)
    message.repeated_uint32_value.append(0xFFFFFFF)
    message.repeated_uint32_value.append(0x7FFFFFF)
    message.repeated_uint64_value.append(9007199254740992)
    message.repeated_uint64_value.append(9007199254740991)
    message.repeated_float_value.append(0)

    message.repeated_double_value.append(1e-15)
    message.repeated_double_value.append(float('inf'))
    message.repeated_bool_value.append(True)
    message.repeated_bool_value.append(False)
    message.repeated_string_value.append('Few symbols!#$,;')
    message.repeated_string_value.append('bar')
    message.repeated_bytes_value.append(b'foo')
    message.repeated_bytes_value.append(b'bar')
    message.repeated_message_value.add().value = 10
    message.repeated_message_value.add().value = 11
    message.repeated_enum_value.append(json_format_proto3_pb2.FOO)
    message.repeated_enum_value.append(json_format_proto3_pb2.BAR)
    self.message = message

  def CheckParseBack(self, message, parsed_message):
    json_format.Parse(json_format.MessageToJson(message), parsed_message)
    self.assertEqual(message, parsed_message)

  def CheckError(self, text, error_message):
    message = json_format_proto3_pb2.TestMessage()
    self.assertRaisesRegex(
        json_format.ParseError, error_message, json_format.Parse, text, message
    )


class JsonFormatTest(JsonFormatBase):

  def testEmptyMessageToJson(self):
    message = json_format_proto3_pb2.TestMessage()
    self.assertEqual(json_format.MessageToJson(message), '{}')
    parsed_message = json_format_proto3_pb2.TestMessage()
    self.CheckParseBack(message, parsed_message)

  def testPartialMessageToJson(self):
    message = json_format_proto3_pb2.TestMessage(
        string_value='test', repeated_int32_value=[89, 4]
    )
    self.assertEqual(
        json.loads(json_format.MessageToJson(message)),
        json.loads('{"stringValue": "test", "repeatedInt32Value": [89, 4]}'),
    )
    parsed_message = json_format_proto3_pb2.TestMessage()
    self.CheckParseBack(message, parsed_message)

  def testAllFieldsToJson(self):
    message = json_format_proto3_pb2.TestMessage()
    text = (
        '{"int32Value": 20, '
        '"int64Value": "-20", '
        '"uint32Value": 3120987654,'
        '"uint64Value": "12345678900",'
        '"floatValue": "-Infinity",'
        '"doubleValue": 3.1415,'
        '"boolValue": true,'
        '"stringValue": "foo",'
        '"bytesValue": "YmFy",'
        '"messageValue": {"value": 10},'
        '"enumValue": "BAR",'
        '"repeatedInt32Value": [2147483647, -2147483648],'
        '"repeatedInt64Value": ["9007199254740992", "-9007199254740992"],'
        '"repeatedUint32Value": [268435455, 134217727],'
        '"repeatedUint64Value": ["9007199254740992", "9007199254740991"],'
        '"repeatedFloatValue": [0],'
        '"repeatedDoubleValue": [1e-15, "Infinity"],'
        '"repeatedBoolValue": [true, false],'
        '"repeatedStringValue": ["Few symbols!#$,;", "bar"],'
        '"repeatedBytesValue": ["Zm9v", "YmFy"],'
        '"repeatedMessageValue": [{"value": 10}, {"value": 11}],'
        '"repeatedEnumValue": ["FOO", "BAR"]'
        '}'
    )
    self.FillAllFields(message)
    self.assertEqual(
        json.loads(json_format.MessageToJson(message)), json.loads(text)
    )
    parsed_message = json_format_proto3_pb2.TestMessage()
    json_format.Parse(text, parsed_message)
    self.assertEqual(message, parsed_message)

  def testUnknownEnumToJsonAndBack(self):
    text = '{\n  "enumValue": 999\n}'
    message = json_format_proto3_pb2.TestMessage()
    message.enum_value = 999
    self.assertEqual(json_format.MessageToJson(message), text)
    parsed_message = json_format_proto3_pb2.TestMessage()
    json_format.Parse(text, parsed_message)
    self.assertEqual(message, parsed_message)

  def testExtensionToJsonAndBack(self):
    message = unittest_mset_pb2.TestMessageSetContainer()
    ext1 = unittest_mset_pb2.TestMessageSetExtension1.message_set_extension
    ext2 = unittest_mset_pb2.TestMessageSetExtension2.message_set_extension
    message.message_set.Extensions[ext1].i = 23
    message.message_set.Extensions[ext2].str = 'foo'
    message_text = json_format.MessageToJson(message)
    parsed_message = unittest_mset_pb2.TestMessageSetContainer()
    json_format.Parse(message_text, parsed_message)
    self.assertEqual(message, parsed_message)

  def testExtensionErrors(self):
    self.CheckError(
        '{"[extensionField]": {}}',
        'Message type proto3.TestMessage does not have extensions',
    )

  def testExtensionToDictAndBack(self):
    message = unittest_mset_pb2.TestMessageSetContainer()
    ext1 = unittest_mset_pb2.TestMessageSetExtension1.message_set_extension
    ext2 = unittest_mset_pb2.TestMessageSetExtension2.message_set_extension
    message.message_set.Extensions[ext1].i = 23
    message.message_set.Extensions[ext2].str = 'foo'
    message_dict = json_format.MessageToDict(message)
    parsed_message = unittest_mset_pb2.TestMessageSetContainer()
    json_format.ParseDict(message_dict, parsed_message)
    self.assertEqual(message, parsed_message)

  def testExtensionToDictAndBackWithScalar(self):
    message = unittest_pb2.TestAllExtensions()
    ext1 = unittest_pb2.TestNestedExtension.test
    message.Extensions[ext1] = 'data'
    message_dict = json_format.MessageToDict(message)
    parsed_message = unittest_pb2.TestAllExtensions()
    json_format.ParseDict(message_dict, parsed_message)
    self.assertEqual(message, parsed_message)

  def testScalarExtensionToDictAndBack(self):
    message = unittest_pb2.TestAllExtensions()
    message.Extensions[unittest_pb2.optional_int32_extension] = 7
    message.Extensions[unittest_pb2.optional_string_extension] = 'hello'
    message_dict = json_format.MessageToDict(message)
    self.assertEqual(
        message_dict,
        {
            '[proto2_unittest.optional_int32_extension]': 7,
            '[proto2_unittest.optional_string_extension]': 'hello',
        },
    )
    parsed_message = unittest_pb2.TestAllExtensions()
    json_format.ParseDict(message_dict, parsed_message)
    self.assertEqual(message, parsed_message)

  def testRepeatedScalarExtensionToDictAndBack(self):
    message = unittest_pb2.TestAllExtensions()
    ext1 = unittest_pb2.repeated_int32_extension
    message.Extensions[ext1].extend([1, 2, 3])
    message_dict = json_format.MessageToDict(message)
    self.assertIn('[proto2_unittest.repeated_int32_extension]', message_dict)
    parsed_message = unittest_pb2.TestAllExtensions()
    json_format.ParseDict(message_dict, parsed_message)
    self.assertEqual(message, parsed_message)

  def testRepeatedMessageExtensionToDictAndBack(self):
    message = unittest_pb2.TestAllExtensions()
    ext1 = unittest_pb2.repeated_nested_message_extension
    sub = unittest_pb2.TestAllTypes.NestedMessage()
    sub.bb = 1
    message.Extensions[ext1].append(sub)
    message_dict = json_format.MessageToDict(message)
    self.assertIn(
        '[proto2_unittest.repeated_nested_message_extension]', message_dict
    )
    parsed_message = unittest_pb2.TestAllExtensions()
    json_format.ParseDict(message_dict, parsed_message)
    self.assertEqual(message, parsed_message)

  def testJsonParseDictToAnyDoesNotAlterInput(self):
    orig_dict = {
        'int32Value': 20,
        '@type': 'type.googleapis.com/proto3.TestMessage',
    }
    copied_dict = json.loads(json.dumps(orig_dict))
    parsed_message = any_pb2.Any()
    json_format.ParseDict(copied_dict, parsed_message)
    self.assertEqual(copied_dict, orig_dict)

  def testExtensionSerializationDictMatchesProto3Spec(self):
    """See go/proto3-json-spec for spec."""
    message = unittest_mset_pb2.TestMessageSetContainer()
    ext1 = unittest_mset_pb2.TestMessageSetExtension1.message_set_extension
    ext2 = unittest_mset_pb2.TestMessageSetExtension2.message_set_extension
    message.message_set.Extensions[ext1].i = 23
    message.message_set.Extensions[ext2].str = 'foo'
    message_dict = json_format.MessageToDict(message)
    golden_dict = {
        'messageSet': {
            '[proto2_unittest.TestMessageSetExtension1.message_set_extension]': {
                'i': 23,
            },
            '[proto2_unittest.TestMessageSetExtension2.message_set_extension]': {
                'str': 'foo',
            },
        },
    }
    self.assertEqual(golden_dict, message_dict)
    parsed_msg = unittest_mset_pb2.TestMessageSetContainer()
    json_format.ParseDict(golden_dict, parsed_msg)
    self.assertEqual(message, parsed_msg)

  def testExtensionSerializationDictMatchesProto3SpecMore(self):
    """See go/proto3-json-spec for spec."""
    message = json_format_pb2.TestMessageWithExtension()
    ext = json_format_pb2.TestExtension.ext
    message.Extensions[ext].value = 'stuff'
    message_dict = json_format.MessageToDict(message)
    expected_dict = {
        '[proto2_unittest.TestExtension.ext]': {
            'value': 'stuff',
        },
    }
    self.assertEqual(expected_dict, message_dict)

  def testExtensionSerializationJsonMatchesProto3Spec(self):
    """See go/proto3-json-spec for spec."""
    message = unittest_mset_pb2.TestMessageSetContainer()
    ext1 = unittest_mset_pb2.TestMessageSetExtension1.message_set_extension
    ext2 = unittest_mset_pb2.TestMessageSetExtension2.message_set_extension
    message.message_set.Extensions[ext1].i = 23
    message.message_set.Extensions[ext2].str = 'foo'
    message_text = json_format.MessageToJson(message)
    ext1_text = 'proto2_unittest.TestMessageSetExtension1.message_set_extension'
    ext2_text = 'proto2_unittest.TestMessageSetExtension2.message_set_extension'
    golden_text = (
        '{"messageSet": {'
        '    "[%s]": {'
        '        "i": 23'
        '    },'
        '    "[%s]": {'
        '        "str": "foo"'
        '    }'
        '}}'
    ) % (ext1_text, ext2_text)
    self.assertEqual(json.loads(golden_text), json.loads(message_text))

  def testJsonEscapeString(self):
    message = json_format_proto3_pb2.TestMessage()
    message.string_value = '&\n<"\r>\b\t\f\\\001/'
    message.string_value += (b'\xe2\x80\xa8\xe2\x80\xa9').decode('utf-8')
    self.assertEqual(
        json_format.MessageToJson(message),
        '{\n  "stringValue": '
        '"&\\n<\\"\\r>\\b\\t\\f\\\\\\u0001/\\u2028\\u2029"\n}',
    )
    parsed_message = json_format_proto3_pb2.TestMessage()
    self.CheckParseBack(message, parsed_message)
    text = '{"int32Value": "\u0031"}'
    json_format.Parse(text, message)
    self.assertEqual(message.int32_value, 1)

  def testProto3Optional_IncludingDefaultValueWithoutPresenceFields(self):
    message = test_proto3_optional_pb2.TestProto3Optional()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads('{"repeatedInt32": [], "repeatedNestedMessage": []}'),
    )
    message.optional_int32 = 0
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{"optionalInt32": 0,"repeatedInt32": [],'
            ' "repeatedNestedMessage": []}'
        ),
    )

  def testProto2_IncludingDefaultValueWithoutPresenceFields(self):
    message = test_proto2_pb2.TestProto2()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads('{"repeatedInt32": [], "repeatedNestedMessage": []}'),
    )
    message.optional_int32 = 0
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{"optionalInt32": 0,"repeatedInt32": [],'
            ' "repeatedNestedMessage": []}'
        ),
    )

  def testIntegersRepresentedAsFloat(self):
    message = json_format_proto3_pb2.TestMessage()
    json_format.Parse('{"int32Value": -2.147483648e9}', message)
    self.assertEqual(message.int32_value, -2147483648)
    json_format.Parse('{"int32Value": 1e5}', message)
    self.assertEqual(message.int32_value, 100000)
    json_format.Parse('{"int32Value": 1.0}', message)
    self.assertEqual(message.int32_value, 1)

  def testIntegersRepresentedAsFloatStrings(self):
    message = json_format_proto3_pb2.TestMessage()
    json_format.Parse('{"int32Value": "-2.147483648e9"}', message)
    self.assertEqual(message.int32_value, -2147483648)
    json_format.Parse('{"int32Value": "1e5"}', message)
    self.assertEqual(message.int32_value, 100000)
    json_format.Parse('{"int32Value": "1.0"}', message)
    self.assertEqual(message.int32_value, 1)

  def testMapFields(self):
    message = json_format_proto3_pb2.TestNestedMap()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{'
            '"boolMap": {},'
            '"int32Map": {},'
            '"int64Map": {},'
            '"uint32Map": {},'
            '"uint64Map": {},'
            '"stringMap": {},'
            '"mapMap": {}'
            '}'
        ),
    )
    message.bool_map[True] = 1
    message.bool_map[False] = 2
    message.int32_map[1] = 2
    message.int32_map[2] = 3
    message.int64_map[1] = 2
    message.int64_map[2] = 3
    message.uint32_map[1] = 2
    message.uint32_map[2] = 3
    message.uint64_map[1] = 2
    message.uint64_map[2] = 3
    message.string_map['1'] = 2
    message.string_map['null'] = 3
    message.map_map['1'].bool_map[True] = 3
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=False
            )
        ),
        json.loads(
            '{'
            '"boolMap": {"false": 2, "true": 1},'
            '"int32Map": {"1": 2, "2": 3},'
            '"int64Map": {"1": 2, "2": 3},'
            '"uint32Map": {"1": 2, "2": 3},'
            '"uint64Map": {"1": 2, "2": 3},'
            '"stringMap": {"1": 2, "null": 3},'
            '"mapMap": {"1": {"boolMap": {"true": 3}}}'
            '}'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestNestedMap()
    self.CheckParseBack(message, parsed_message)

  def testOneofFields(self):
    message = json_format_proto3_pb2.TestOneof()
    # Always print does not affect oneof fields.
    self.assertEqual(
        json_format.MessageToJson(
            message, always_print_fields_with_no_presence=True
        ),
        '{}',
    )
    message.oneof_int32_value = 0
    self.assertEqual(
        json_format.MessageToJson(
            message, always_print_fields_with_no_presence=True
        ),
        '{\n  "oneofInt32Value": 0\n}',
    )
    parsed_message = json_format_proto3_pb2.TestOneof()
    self.CheckParseBack(message, parsed_message)

  def testSurrogates(self):
    # Test correct surrogate handling.
    message = json_format_proto3_pb2.TestMessage()
    json_format.Parse('{"stringValue": "\\uD83D\\uDE01"}', message)
    self.assertEqual(
        message.string_value, b'\xF0\x9F\x98\x81'.decode('utf-8', 'strict')
    )

    # Error case: unpaired high surrogate.
    self.CheckError(
        '{"stringValue": "\\uD83D"}',
        r'Invalid \\uXXXX escape|Unpaired.*surrogate',
    )

    # Unpaired low surrogate.
    self.CheckError(
        '{"stringValue": "\\uDE01"}',
        r'Invalid \\uXXXX escape|Unpaired.*surrogate',
    )

  def testTimestampMessage(self):
    message = json_format_proto3_pb2.TestTimestamp()
    message.value.seconds = 0
    message.value.nanos = 0
    message.repeated_value.add().seconds = 20
    message.repeated_value[0].nanos = 1
    message.repeated_value.add().seconds = 0
    message.repeated_value[1].nanos = 10000
    message.repeated_value.add().seconds = 100000000
    message.repeated_value[2].nanos = 0
    # Maximum time
    message.repeated_value.add().seconds = 253402300799
    message.repeated_value[3].nanos = 999999999
    # Minimum time
    message.repeated_value.add().seconds = -62135596800
    message.repeated_value[4].nanos = 0
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{'
            '"value": "1970-01-01T00:00:00Z",'
            '"repeatedValue": ['
            '  "1970-01-01T00:00:20.000000001Z",'
            '  "1970-01-01T00:00:00.000010Z",'
            '  "1973-03-03T09:46:40Z",'
            '  "9999-12-31T23:59:59.999999999Z",'
            '  "0001-01-01T00:00:00Z"'
            ']'
            '}'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestTimestamp()
    self.CheckParseBack(message, parsed_message)
    text = (
        r'{"value": "1970-01-01T00:00:00.01+08:00",'
        r'"repeatedValue":['
        r'  "1970-01-01T00:00:00.01+08:30",'
        r'  "1970-01-01T00:00:00.01-01:23"]}'
    )
    json_format.Parse(text, parsed_message)
    self.assertEqual(parsed_message.value.seconds, -8 * 3600)
    self.assertEqual(parsed_message.value.nanos, 10000000)
    self.assertEqual(parsed_message.repeated_value[0].seconds, -8.5 * 3600)
    self.assertEqual(parsed_message.repeated_value[1].seconds, 3600 + 23 * 60)

  def testDurationMessage(self):
    message = json_format_proto3_pb2.TestDuration()
    message.value.seconds = 1
    message.repeated_value.add().seconds = 0
    message.repeated_value[0].nanos = 10
    message.repeated_value.add().seconds = -1
    message.repeated_value[1].nanos = -1000
    message.repeated_value.add().seconds = 10
    message.repeated_value[2].nanos = 11000000
    message.repeated_value.add().seconds = -315576000000
    message.repeated_value.add().seconds = 315576000000
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{'
            '"value": "1s",'
            '"repeatedValue": ['
            '  "0.000000010s",'
            '  "-1.000001s",'
            '  "10.011s",'
            '  "-315576000000s",'
            '  "315576000000s"'
            ']'
            '}'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestDuration()
    self.CheckParseBack(message, parsed_message)

  def testFieldMaskMessage(self):
    message = json_format_proto3_pb2.TestFieldMask()
    message.value.paths.append('foo.bar')
    message.value.paths.append('bar')
    self.assertEqual(
        json_format.MessageToJson(
            message, always_print_fields_with_no_presence=True
        ),
        '{\n  "value": "foo.bar,bar"\n}',
    )
    parsed_message = json_format_proto3_pb2.TestFieldMask()
    self.CheckParseBack(message, parsed_message)

    message.value.Clear()
    self.assertEqual(
        json_format.MessageToJson(
            message, always_print_fields_with_no_presence=True
        ),
        '{\n  "value": ""\n}',
    )
    self.CheckParseBack(message, parsed_message)

  def testWrapperMessage(self):
    message = json_format_proto3_pb2.TestWrapper()
    message.bool_value.value = False
    message.int32_value.value = 0
    message.string_value.value = ''
    message.bytes_value.value = b''
    message.repeated_bool_value.add().value = True
    message.repeated_bool_value.add().value = False
    message.repeated_int32_value.add()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "int32Value": 0,'
            '  "boolValue": false,'
            '  "stringValue": "",'
            '  "bytesValue": "",'
            '  "repeatedBoolValue": [true, false],'
            '  "repeatedInt32Value": [0],'
            '  "repeatedUint32Value": [],'
            '  "repeatedFloatValue": [],'
            '  "repeatedDoubleValue": [],'
            '  "repeatedBytesValue": [],'
            '  "repeatedInt64Value": [],'
            '  "repeatedUint64Value": [],'
            '  "repeatedStringValue": []'
            '}'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestWrapper()
    self.CheckParseBack(message, parsed_message)

  def testStructMessage(self):
    message = json_format_proto3_pb2.TestStruct()
    message.value['name'] = 'Jim'
    message.value['age'] = 10
    message.value['attend'] = True
    message.value['email'] = None
    message.value.get_or_create_struct('address')['city'] = 'SFO'
    message.value['address']['house_number'] = 1024
    message.value.get_or_create_struct('empty_struct')
    message.value.get_or_create_list('empty_list')
    struct_list = message.value.get_or_create_list('list')
    struct_list.extend([6, 'seven', True, False, None])
    struct_list.add_struct()['subkey2'] = 9
    message.repeated_value.add()['age'] = 11
    message.repeated_value.add()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=False
            )
        ),
        json.loads(
            '{'
            '  "value": {'
            '    "address": {'
            '      "city": "SFO", '
            '      "house_number": 1024'
            '    }, '
            '    "empty_struct": {}, '
            '    "empty_list": [], '
            '    "age": 10, '
            '    "name": "Jim", '
            '    "attend": true, '
            '    "email": null, '
            '    "list": [6, "seven", true, false, null, {"subkey2": 9}]'
            '  },'
            '  "repeatedValue": [{"age": 11}, {}]'
            '}'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestStruct()
    self.CheckParseBack(message, parsed_message)
    # check for regression; this used to raise
    _ = parsed_message.value['empty_struct']
    _ = parsed_message.value['empty_list']

  def testValueMessage(self):
    message = json_format_proto3_pb2.TestValue()
    message.value.string_value = 'hello'
    message.repeated_value.add().number_value = 11.1
    message.repeated_value.add().bool_value = False
    message.repeated_value.add().null_value = 0
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=False
            )
        ),
        json.loads(
            '{  "value": "hello",  "repeatedValue": [11.1, false, null]}'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestValue()
    self.CheckParseBack(message, parsed_message)
    # Can't parse back if the Value message is not set.
    message.repeated_value.add()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=False
            )
        ),
        json.loads(
            '{  "value": "hello",  "repeatedValue": [11.1, false, null, null]}'
        ),
    )
    message.Clear()
    json_format.Parse('{"value": null}', message)
    self.assertEqual(message.value.WhichOneof('kind'), 'null_value')

  def testValueMessageErrors(self):
    message = json_format_proto3_pb2.TestValue()
    message.value.number_value = math.inf
    with self.assertRaises(json_format.SerializeToJsonError) as cm:
      json_format.MessageToJson(message)
    self.assertEqual(
        'Failed to serialize value field: Fail to serialize Infinity for '
        'Value.number_value, which would parse as string_value.',
        str(cm.exception),
    )
    message.value.number_value = math.nan
    with self.assertRaises(json_format.SerializeToJsonError) as cm:
      json_format.MessageToJson(message)
    self.assertEqual(
        'Failed to serialize value field: Fail to serialize NaN for '
        'Value.number_value, which would parse as string_value.',
        str(cm.exception),
    )

  def testListValueMessage(self):
    message = json_format_proto3_pb2.TestListValue()
    message.value.values.add().number_value = 11.1
    message.value.values.add().null_value = 0
    message.value.values.add().bool_value = True
    message.value.values.add().string_value = 'hello'
    message.value.values.add().struct_value['name'] = 'Jim'
    message.repeated_value.add().values.add().number_value = 1
    message.repeated_value.add()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=False
            )
        ),
        json.loads(
            '{"value": [11.1, null, true, "hello", {"name": "Jim"}]\n,'
            '"repeatedValue": [[1], []]}'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestListValue()
    self.CheckParseBack(message, parsed_message)

  def testNullValue(self):
    message = json_format_proto3_pb2.TestOneof()
    message.oneof_null_value = 0
    self.assertEqual(
        json_format.MessageToJson(message), '{\n  "oneofNullValue": null\n}'
    )
    parsed_message = json_format_proto3_pb2.TestOneof()
    self.CheckParseBack(message, parsed_message)
    # Check old format is also accepted
    new_message = json_format_proto3_pb2.TestOneof()
    json_format.Parse('{\n  "oneofNullValue": "NULL_VALUE"\n}', new_message)
    self.assertEqual(
        json_format.MessageToJson(new_message), '{\n  "oneofNullValue": null\n}'
    )

  def testAnyMessage(self):
    message = json_format_proto3_pb2.TestAny()
    value1 = json_format_proto3_pb2.MessageType()
    value2 = json_format_proto3_pb2.MessageType()
    value1.value = 1234
    value2.value = 5678
    message.value.Pack(value1)
    message.repeated_value.add().Pack(value1)
    message.repeated_value.add().Pack(value2)
    message.repeated_value.add()
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "repeatedValue": [ {\n'
            '    "@type": "type.googleapis.com/proto3.MessageType",\n'
            '    "value": 1234\n'
            '  }, {\n'
            '    "@type": "type.googleapis.com/proto3.MessageType",\n'
            '    "value": 5678\n'
            '  },\n'
            '  {}],\n'
            '  "value": {\n'
            '    "@type": "type.googleapis.com/proto3.MessageType",\n'
            '    "value": 1234\n'
            '  }\n'
            '}\n'
        ),
    )
    parsed_message = json_format_proto3_pb2.TestAny()
    self.CheckParseBack(message, parsed_message)
    # Must print @type first
    test_message = json_format_proto3_pb2.TestMessage(
        bool_value=True,
        int32_value=20,
        int64_value=-20,
        uint32_value=20,
        uint64_value=20,
        double_value=3.14,
        string_value='foo',
    )
    message.Clear()
    message.value.Pack(test_message)
    self.assertEqual(
        json_format.MessageToJson(
            message, always_print_fields_with_no_presence=False
        )[0:68],
        '{\n'
        '  "value": {\n'
        '    "@type": "type.googleapis.com/proto3.TestMessage"',
    )

  def testAnyMessageDescriptorPoolMissingType(self):
    packed_message = unittest_pb2.OneString()
    packed_message.data = 'string'
    message = any_test_pb2.TestAny()
    message.any_value.Pack(packed_message)
    empty_pool = descriptor_pool.DescriptorPool()
    with self.assertRaises(TypeError) as cm:
      json_format.MessageToJson(
          message,
          always_print_fields_with_no_presence=True,
          descriptor_pool=empty_pool,
      )
    self.assertEqual(
        'Can not find message descriptor by type_url:'
        ' type.googleapis.com/proto2_unittest.OneString',
        str(cm.exception),
    )

  def testWellKnownInAnyMessage(self):
    message = any_pb2.Any()
    int32_value = wrappers_pb2.Int32Value()
    int32_value.value = 1234
    message.Pack(int32_value)
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "@type": "type.googleapis.com/google.protobuf.Int32Value",\n'
            '  "value": 1234\n'
            '}\n'
        ),
    )
    parsed_message = any_pb2.Any()
    self.CheckParseBack(message, parsed_message)

    timestamp = timestamp_pb2.Timestamp()
    message.Pack(timestamp)
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "@type": "type.googleapis.com/google.protobuf.Timestamp",\n'
            '  "value": "1970-01-01T00:00:00Z"\n'
            '}\n'
        ),
    )
    self.CheckParseBack(message, parsed_message)

    duration = duration_pb2.Duration()
    duration.seconds = 1
    message.Pack(duration)
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "@type": "type.googleapis.com/google.protobuf.Duration",\n'
            '  "value": "1s"\n'
            '}\n'
        ),
    )
    self.CheckParseBack(message, parsed_message)

    field_mask = field_mask_pb2.FieldMask()
    field_mask.paths.append('foo.bar')
    field_mask.paths.append('bar')
    message.Pack(field_mask)
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "@type": "type.googleapis.com/google.protobuf.FieldMask",\n'
            '  "value": "foo.bar,bar"\n'
            '}\n'
        ),
    )
    self.CheckParseBack(message, parsed_message)

    struct_message = struct_pb2.Struct()
    struct_message['name'] = 'Jim'
    message.Pack(struct_message)
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "@type": "type.googleapis.com/google.protobuf.Struct",\n'
            '  "value": {"name": "Jim"}\n'
            '}\n'
        ),
    )
    self.CheckParseBack(message, parsed_message)

    nested_any = any_pb2.Any()
    int32_value.value = 5678
    nested_any.Pack(int32_value)
    message.Pack(nested_any)
    self.assertEqual(
        json.loads(
            json_format.MessageToJson(
                message, always_print_fields_with_no_presence=True
            )
        ),
        json.loads(
            '{\n'
            '  "@type": "type.googleapis.com/google.protobuf.Any",\n'
            '  "value": {\n'
            '    "@type": "type.googleapis.com/google.protobuf.Int32Value",\n'
            '    "value": 5678\n'
            '  }\n'
            '}\n'
        ),
    )
    self.CheckParseBack(message, parsed_message)

  def testParseNull(self):
    message = json_format_proto3_pb2.TestMessage()
    parsed_message = json_format_proto3_pb2.TestMessage()
    self.FillAllFields(parsed_message)
    json_format.Parse(
        '{"int32Value": null, '
        '"int64Value": null, '
        '"uint32Value": null,'
        '"uint64Value": null,'
        '"floatValue": null,'
        '"doubleValue": null,'
        '"boolValue": null,'
        '"stringValue": null,'
        '"bytesValue": null,'
        '"messageValue": null,'
        '"enumValue": null,'
        '"repeatedInt32Value": null,'
        '"repeatedInt64Value": null,'
        '"repeatedUint32Value": null,'
        '"repeatedUint64Value": null,'
        '"repeatedFloatValue": null,'
        '"repeatedDoubleValue": null,'
        '"repeatedBoolValue": null,'
        '"repeatedStringValue": null,'
        '"repeatedBytesValue": null,'
        '"repeatedMessageValue": null,'
        '"repeatedEnumValue": null'
        '}',
        parsed_message,
    )
    self.assertEqual(message, parsed_message)
    # Null and {} should have different behavior for sub message.
    self.assertFalse(parsed_message.HasField('message_value'))
    json_format.Parse('{"messageValue": {}}', parsed_message)
    self.assertTrue(parsed_message.HasField('message_value'))
    # Null is not allowed to be used as an element in repeated field.
    self.assertRaisesRegex(
        json_format.ParseError,
        r'Failed to parse repeatedInt32Value field: '
        r'null is not allowed to be used as an element in a repeated field '
        r'at TestMessage.repeatedInt32Value\[1\].',
        json_format.Parse,
        '{"repeatedInt32Value":[1, null]}',
        parsed_message,
    )
    self.CheckError(
        '{"repeatedMessageValue":[null]}',
        r'Failed to parse repeatedMessageValue field: null is not'
        r' allowed to be used as an element in a repeated field '
        r'at TestMessage.repeatedMessageValue\[0\].',
    )

  def testNanFloat(self):
    message = json_format_proto3_pb2.TestMessage()
    message.float_value = float('nan')
    text = '{\n  "floatValue": "NaN"\n}'
    self.assertEqual(json_format.MessageToJson(message), text)
    parsed_message = json_format_proto3_pb2.TestMessage()
    json_format.Parse(text, parsed_message)
    self.assertTrue(math.isnan(parsed_message.float_value))

  def testParseDoubleToFloat(self):
    message = json_format_proto3_pb2.TestMessage()
    text = '{"repeatedDoubleValue": [3.4028235e+39, 1.4028235e-39]\n}'
    json_format.Parse(text, message)
    self.assertEqual(message.repeated_double_value[0], 3.4028235e39)
    self.assertEqual(message.repeated_double_value[1], 1.4028235e-39)
    text = '{"repeatedFloatValue": [3.4028235e+39, 1.4028235e-39]\n}'
    self.CheckError(
        text,
        r'Failed to parse repeatedFloatValue field: '
        r'Float value too large at TestMessage.repeatedFloatValue\[0\].',
    )

  def testFloatPrecision(self):
    message = json_format_proto3_pb2.TestMessage()

    # Default float_precision will automatic print shortest float.
    message.float_value = 1.1000000011
    text = '{\n  "floatValue": 1.1\n}'
    self.assertEqual(json_format.MessageToJson(message), text)
    message.float_value = 1.00000075e-36
    text = '{\n  "floatValue": 1.00000075e-36\n}'
    self.assertEqual(json_format.MessageToJson(message), text)
    message.float_value = 12345678912345e11
    text = '{\n  "floatValue": 1.234568e+24\n}'
    self.assertEqual(json_format.MessageToJson(message), text)

    # Test a bunch of data and check json encode/decode do not
    # lose precision
    value_list = [0x00, 0xD8, 0x6E, 0x00]
    msg2 = json_format_proto3_pb2.TestMessage()
    for a in range(0, 256):
      value_list[3] = a
      for b in range(0, 256):
        value_list[0] = b
        byte_array = bytearray(value_list)
        message.float_value = struct.unpack('<f', byte_array)[0]
        self.CheckParseBack(message, msg2)

  def testParseEmptyText(self):
    self.CheckError('', r'Failed to load JSON: (Expecting value)|(No JSON).')

  def testParseEnumValue(self):
    message = json_format_proto3_pb2.TestMessage()
    text = '{"enumValue": 0}'
    json_format.Parse(text, message)
    text = '{"enumValue": 1}'
    json_format.Parse(text, message)
    self.CheckError(
        '{"enumValue": "baz"}',
        'Failed to parse enumValue field: Invalid enum value baz '
        'for enum type proto3.EnumType at TestMessage.enumValue.',
    )
    # Proto3 accepts numeric unknown enums.
    text = '{"enumValue": 12345}'
    json_format.Parse(text, message)
    # Proto2 does not accept numeric unknown enums.
    message = unittest_pb2.TestAllTypes()
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to parse optionalNestedEnum field: Invalid enum value 12345 '
        'for enum type proto2_unittest.TestAllTypes.NestedEnum at '
        'TestAllTypes.optionalNestedEnum.',
        json_format.Parse,
        '{"optionalNestedEnum": 12345}',
        message,
    )

  def testParseUnknownEnumStringValue_Scalar_Proto2(self):
    message = json_format_pb2.TestNumbers()
    text = '{"a": "UNKNOWN_STRING_VALUE"}'
    json_format.Parse(text, message, ignore_unknown_fields=True)

    self.assertFalse(message.HasField('a'))

  def testParseErrorForUnknownEnumValue_ScalarWithoutIgnore_Proto2(self):
    message = json_format_pb2.TestNumbers()
    self.assertRaisesRegex(
        json_format.ParseError,
        'Invalid enum value',
        json_format.Parse,
        '{"a": "UNKNOWN_STRING_VALUE"}',
        message,
    )

  def testParseUnknownEnumStringValue_Repeated_Proto2(self):
    message = json_format_pb2.TestRepeatedEnum()
    text = '{"repeatedEnum": ["UNKNOWN_STRING_VALUE", "BUFFER"]}'
    json_format.Parse(text, message, ignore_unknown_fields=True)

    self.assertEqual(len(message.repeated_enum), 1)
    self.assertTrue(message.repeated_enum[0] == json_format_pb2.BUFFER)

  def testParseUnknownEnumStringValue_Map_Proto2(self):
    message = json_format_pb2.TestMapOfEnums()
    text = '{"enumMap": {"key1": "BUFFER", "key2": "UNKNOWN_STRING_VALUE"}}'
    json_format.Parse(text, message, ignore_unknown_fields=True)

    self.assertTrue(message.enum_map['key1'] == json_format_pb2.BUFFER)
    self.assertFalse('key2' in message.enum_map)

  def testParseUnknownEnumStringValue_ExtensionField_Proto2(self):
    message = json_format_pb2.TestMessageWithExtension()
    text = """
        {"[proto2_unittest.TestExtension.enum_ext]": "UNKNOWN_STRING_VALUE"}
    """
    json_format.Parse(text, message, ignore_unknown_fields=True)

    self.assertFalse(
        json_format_pb2.TestExtension.enum_ext in message.Extensions
    )

  def testParseUnknownEnumStringValue_ExtensionFieldWithoutIgnore_Proto2(self):
    message = json_format_pb2.TestMessageWithExtension()
    text = """
        {"[proto2_unittest.TestExtension.enum_ext]": "UNKNOWN_STRING_VALUE"}
    """
    self.assertRaisesRegex(
        json_format.ParseError,
        'Invalid enum value',
        json_format.Parse,
        text,
        message,
    )

  def testParseUnknownEnumStringValue_Scalar_Proto3(self):
    message = json_format_proto3_pb2.TestMessage()
    text = '{"enumValue": "UNKNOWN_STRING_VALUE"}'

    json_format.Parse(text, message, ignore_unknown_fields=True)
    self.assertEqual(message.enum_value, 0)

  def testParseUnknownEnumStringValue_Repeated_Proto3(self):
    message = json_format_proto3_pb2.TestMessage()
    text = '{"repeatedEnumValue": ["UNKNOWN_STRING_VALUE", "FOO"]}'
    json_format.Parse(text, message, ignore_unknown_fields=True)

    self.assertEqual(len(message.repeated_enum_value), 1)
    self.assertTrue(
        message.repeated_enum_value[0] == json_format_proto3_pb2.FOO
    )

  def testParseUnknownEnumStringValue_Map_Proto3(self):
    message = json_format_proto3_pb2.MapOfEnums()
    text = '{"map": {"key1": "FOO", "key2": "UNKNOWN_STRING_VALUE"}}'
    json_format.Parse(text, message, ignore_unknown_fields=True)

    self.assertTrue(message.map['key1'] == json_format_proto3_pb2.FOO)
    self.assertFalse('key2' in message.map)

  def testBytes(self):
    message = json_format_proto3_pb2.TestMessage()
    # Test url base64
    text = '{"bytesValue": "-_"}'
    json_format.Parse(text, message)
    self.assertEqual(message.bytes_value, b'\xfb')
    # Test padding
    text = '{"bytesValue": "AQI="}'
    json_format.Parse(text, message)
    self.assertEqual(message.bytes_value, b'\x01\x02')
    text = '{"bytesValue": "AQI"}'
    json_format.Parse(text, message)
    self.assertEqual(message.bytes_value, b'\x01\x02')
    text = '{"bytesValue": "AQI*"}'
    json_format.Parse(text, message)
    self.assertEqual(message.bytes_value, b'\x01\x02')

  def testParseBadIdentifier(self):
    self.CheckError(
        '{int32Value: 1}',
        (
            r'Failed to load JSON: Expecting property name'
            r'( enclosed in double quotes)?: line 1'
        ),
    )
    self.CheckError(
        '{"unknownName": 1}',
        'Message type "proto3.TestMessage" has no field named '
        '"unknownName" at "TestMessage".',
    )

  def testIgnoreUnknownField(self):
    text = '{"unknownName": 1}'
    parsed_message = json_format_proto3_pb2.TestMessage()
    json_format.Parse(text, parsed_message, ignore_unknown_fields=True)
    text = (
        '{\n'
        '  "repeatedValue": [ {\n'
        '    "@type": "type.googleapis.com/proto3.MessageType",\n'
        '    "unknownName": 1\n'
        '  }]\n'
        '}\n'
    )
    parsed_message = json_format_proto3_pb2.TestAny()
    json_format.Parse(text, parsed_message, ignore_unknown_fields=True)

  def testDuplicateField(self):
    self.CheckError(
        '{"int32Value": 1,\n"int32Value":2}',
        'Failed to load JSON: duplicate key int32Value.',
    )

  def testDuplicateFieldAlternateNames(self):
    # Note: this behavior is non-spec and an oversight bug in the
    # implementation, but would be a breaking change to fix. The duplicate field
    # checker intends reject inputs with duplicate key names, but it only
    # catches keys that are exact matches and not alternate spellings that
    # correspond to the same field.
    parsed_message = json_format_proto3_pb2.TestMessage()
    json_format.Parse('{"int32Value": 1,"int32_value":2}', parsed_message)
    self.assertEqual(parsed_message.int32_value, 2)

  def testDuplicateFieldAlternateNamesMap(self):
    # Note: this behavior is non-spec and an oversight bug in the
    # implementation, but would be a breaking change to fix. The duplicate field
    # checker intends reject inputs with duplicate key names, but it only
    # catches keys that are exact matches and not alternate spellings that
    # correspond to the same field.
    parsed_message = json_format_proto3_pb2.TestMap()
    json_format.Parse(
        '{"int32Map": {"1": 2}, "int32_map": {"3": 4}}', parsed_message
    )
    self.assertEqual(parsed_message.int32_map, {3: 4})

  def testInvalidBoolValue(self):
    self.CheckError(
        '{"boolValue": 1}',
        'Failed to parse boolValue field: '
        'Expected true or false without quotes at TestMessage.boolValue.',
    )
    self.CheckError(
        '{"boolValue": "true"}',
        'Failed to parse boolValue field: '
        'Expected true or false without quotes at TestMessage.boolValue.',
    )

  def testInvalidIntegerValue(self):
    message = json_format_proto3_pb2.TestMessage()
    text = '{"int32Value": 0x12345}'
    self.assertRaises(json_format.ParseError, json_format.Parse, text, message)
    self.CheckError(
        '{"int32Value": 1.5}',
        'Failed to parse int32Value field: '
        "Couldn't parse integer: 1.5 at TestMessage.int32Value.",
    )
    self.CheckError(
        '{"int32Value": "1.5"}',
        'Failed to parse int32Value field: '
        'Couldn\'t parse non-integer string: "1.5" at TestMessage.int32Value.',
    )
    self.CheckError(
        '{"int32Value": "foo"}',
        'Failed to parse int32Value field: invalid literal for int\(\) with'
        " base 10: 'foo'.",
    )
    self.CheckError(
        '{"int32Value": 012345}',
        (r'Failed to load JSON: Expecting \'?,\'? delimiter: ' r'line 1.'),
    )
    self.CheckError(
        '{"int32Value": " 1 "}',
        'Failed to parse int32Value field: '
        'Couldn\'t parse integer: " 1 " at TestMessage.int32Value.',
    )
    self.CheckError(
        '{"int32Value": "1 "}',
        'Failed to parse int32Value field: '
        'Couldn\'t parse integer: "1 " at TestMessage.int32Value.',
    )
    self.CheckError(
        '{"int32Value": false}',
        'Failed to parse int32Value field: Bool value False '
        'is not acceptable for integer field at TestMessage.int32Value.',
    )
    self.CheckError(
        '{"int32Value": 12345678901234567890}',
        'Failed to parse int32Value field: Value out of range: '
        '12345678901234567890.',
    )
    self.CheckError(
        '{"uint32Value": -1}',
        'Failed to parse uint32Value field: Value out of range: -1.',
    )

  def testInvalidFloatValue(self):
    self.CheckError(
        '{"floatValue": "nan"}',
        "Failed to parse floatValue field: Couldn't "
        'parse float "nan", use "NaN" instead at TestMessage.floatValue.',
    )
    self.CheckError(
        '{"floatValue": NaN}',
        "Failed to parse floatValue field: Couldn't "
        'parse NaN, use quoted "NaN" instead.',
    )
    self.CheckError(
        '{"floatValue": Infinity}',
        "Failed to parse floatValue field: Couldn't parse Infinity"
        ' or value too large, use quoted "Infinity" instead.',
    )
    self.CheckError(
        '{"floatValue": -Infinity}',
        "Failed to parse floatValue field: Couldn't parse "
        '-Infinity or value too small, '
        'use quoted "-Infinity" instead.',
    )
    self.CheckError(
        '{"doubleValue": -1.89769e+308}',
        "Failed to parse doubleValue field: Couldn't parse "
        '-Infinity or value too small, '
        'use quoted "-Infinity" instead.',
    )
    self.CheckError(
        '{"floatValue": 3.4028235e+39}',
        'Failed to parse floatValue field: Float value too large.',
    )
    self.CheckError(
        '{"floatValue": -3.502823e+38}',
        'Failed to parse floatValue field: Float value too small.',
    )

  def testInvalidRepeated(self):
    self.CheckError(
        '{"repeatedInt32Value": 12345}',
        (
            r'Failed to parse repeatedInt32Value field: repeated field'
            r' repeatedInt32Value must be in \[\] which is 12345 at'
            r' TestMessage.'
        ),
    )

  def testInvalidMap(self):
    message = json_format_proto3_pb2.TestMap()
    text = '{"int32Map": {"null": 2, "2": 3}}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to parse int32Map field: invalid literal',
        json_format.Parse,
        text,
        message,
    )
    text = '{"int32Map": {1: 2, "2": 3}}'
    self.assertRaisesRegex(
        json_format.ParseError,
        (
            r'Failed to load JSON: Expecting property name'
            r'( enclosed in double quotes)?: line 1'
        ),
        json_format.Parse,
        text,
        message,
    )
    text = '{"boolMap": {"null": 1}}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to parse boolMap field: Expected "true" or "false", not null at'
        ' TestMap.boolMap.key',
        json_format.Parse,
        text,
        message,
    )
    text = r'{"stringMap": {"a": 3, "\u0061": 2}}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to load JSON: duplicate key a',
        json_format.Parse,
        text,
        message,
    )
    text = r'{"stringMap": 0}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to parse stringMap field: Map field string_map must be '
        'in a dict which is 0 at TestMap.stringMap.',
        json_format.Parse,
        text,
        message,
    )

  def testInvalidTimestamp(self):
    message = json_format_proto3_pb2.TestTimestamp()
    text = '{"value": "10000-01-01T00:00:00.00Z"}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to parse value field: '
        "time data '10000-01-01T00:00:00' does not match"
        " format '%Y-%m-%dT%H:%M:%S' at TestTimestamp.value.",
        json_format.Parse,
        text,
        message,
    )
    text = '{"value": "1970-01-01T00:00:00.0123456789012Z"}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'nanos 0123456789012 more than 9 fractional digits.',
        json_format.Parse,
        text,
        message,
    )
    text = '{"value": "1972-01-01T01:00:00.01+08"}'
    self.assertRaisesRegex(
        json_format.ParseError,
        r'Invalid timezone offset value: \+08.',
        json_format.Parse,
        text,
        message,
    )
    # Time smaller than minimum time.
    text = '{"value": "0000-01-01T00:00:00Z"}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to parse value field: year ',
        json_format.Parse,
        text,
        message,
    )
    # Time bigger than maximum time.
    message.value.seconds = 253402300800
    self.assertRaisesRegex(json_format.SerializeToJsonError,
                           'Timestamp is not valid',
                           json_format.MessageToJson, message)
    # Nanos smaller than 0
    message.value.seconds = 0
    message.value.nanos = -1
    self.assertRaisesRegex(
        json_format.SerializeToJsonError,
        'Timestamp is not valid',
        json_format.MessageToJson,
        message,
    )
    # Lower case t does not accept.
    text = '{"value": "0001-01-01t00:00:00Z"}'
    with self.assertRaises(json_format.ParseError) as e:
      json_format.Parse(text, message)
    self.assertEqual(
        'Failed to parse value field: '
        "time data '0001-01-01t00:00:00' does not match format "
        "'%Y-%m-%dT%H:%M:%S', lowercase 't' is not accepted "
        'at TestTimestamp.value.',
        str(e.exception),
    )

  def testInvalidOneof(self):
    message = json_format_proto3_pb2.TestOneof()
    text = '{"oneofInt32Value": 1, "oneofStringValue": "2"}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Message type "proto3.TestOneof"'
        ' should not have multiple "oneof_value" oneof fields at "TestOneof".',
        json_format.Parse,
        text,
        message,
    )

  def testInvalidListValue(self):
    message = json_format_proto3_pb2.TestListValue()
    text = '{"value": 1234}'
    self.assertRaisesRegex(
        json_format.ParseError,
        r'Failed to parse value field: ListValue must be in \[\] which is '
        '1234 at TestListValue.value.',
        json_format.Parse,
        text,
        message,
    )

    class UnknownClass(object):

      def __str__(self):
        return 'v'

    self.assertRaisesRegex(
        json_format.ParseError,
        r' at TestListValue.value\[1\].fake.',
        json_format.ParseDict,
        {'value': ['hello', {'fake': UnknownClass()}]},
        message,
    )

  def testInvalidStruct(self):
    message = json_format_proto3_pb2.TestStruct()
    text = '{"value": 1234}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Failed to parse value field: Struct must be in a dict which is '
        '1234 at TestStruct.value',
        json_format.Parse,
        text,
        message,
    )

  def testTimestampInvalidStringValue(self):
    message = json_format_proto3_pb2.TestTimestamp()
    text = '{"value": {"foo": 123}}'
    self.assertRaisesRegex(
        json_format.ParseError,
        r"Timestamp JSON value not a string: {u?'foo': 123}",
        json_format.Parse,
        text,
        message,
    )

  def testDurationInvalidStringValue(self):
    message = json_format_proto3_pb2.TestDuration()
    text = '{"value": {"foo": 123}}'
    self.assertRaisesRegex(
        json_format.ParseError,
        r"Duration JSON value not a string: {u?'foo': 123}",
        json_format.Parse,
        text,
        message,
    )

  def testFieldMaskInvalidStringValue(self):
    message = json_format_proto3_pb2.TestFieldMask()
    text = '{"value": {"foo": 123}}'
    self.assertRaisesRegex(
        json_format.ParseError,
        r"FieldMask JSON value not a string: {u?'foo': 123}",
        json_format.Parse,
        text,
        message,
    )

  def testInvalidAny(self):
    message = any_pb2.Any()
    text = '{"@type": "type.googleapis.com/google.protobuf.Int32Value"}'
    self.assertRaisesRegex(
        json_format.ParseError,
        "KeyError: 'value'",
        json_format.Parse,
        text,
        message,
    )
    text = '{"value": 1234}'
    self.assertRaisesRegex(
        json_format.ParseError,
        '@type is missing when parsing any message at Any',
        json_format.Parse,
        text,
        message,
    )
    text = '{"@type": "type.googleapis.com/MessageNotExist", "value": 1234}'
    self.assertRaisesRegex(
        json_format.ParseError,
        'Can not find message descriptor by type_url: '
        'type.googleapis.com/MessageNotExist at Any',
        json_format.Parse,
        text,
        message,
    )
    # Only last part is to be used: b/25630112
    text = (
        r'{"@type": "incorrect.googleapis.com/google.protobuf.Int32Value",'
        r'"value": 1234}'
    )
    json_format.Parse(text, message)

  def testPreservingProtoFieldNames(self):
    message = json_format_proto3_pb2.TestMessage()
    message.int32_value = 12345
    self.assertEqual(
        '{\n  "int32Value": 12345\n}', json_format.MessageToJson(message)
    )
    self.assertEqual(
        '{\n  "int32_value": 12345\n}',
        json_format.MessageToJson(
            message,
            always_print_fields_with_no_presence=False,
            preserving_proto_field_name=True,
        ),
    )
    # When always_print_fields_with_no_presence is True.
    message = json_format_proto3_pb2.TestTimestamp()
    self.assertEqual(
        '{\n  "repeatedValue": []\n}',
        json_format.MessageToJson(
            message,
            always_print_fields_with_no_presence=True,
        ),
    )
    self.assertEqual(
        '{\n  "repeated_value": []\n}',
        json_format.MessageToJson(
            message,
            always_print_fields_with_no_presence=True,
            preserving_proto_field_name=True,
        ),
    )

    # Parsers accept both original proto field names and lowerCamelCase names.
    message = json_format_proto3_pb2.TestMessage()
    json_format.Parse('{"int32Value": 54321}', message)
    self.assertEqual(54321, message.int32_value)
    json_format.Parse('{"int32_value": 12345}', message)
    self.assertEqual(12345, message.int32_value)

  def testIndent(self):
    message = json_format_proto3_pb2.TestMessage()
    message.int32_value = 12345
    self.assertEqual(
        '{\n"int32Value": 12345\n}',
        json_format.MessageToJson(message, indent=0),
    )

  def testFormatEnumsAsInts(self):
    message = json_format_proto3_pb2.TestMessage()
    message.enum_value = json_format_proto3_pb2.BAR
    message.repeated_enum_value.append(json_format_proto3_pb2.FOO)
    message.repeated_enum_value.append(json_format_proto3_pb2.BAR)
    self.assertEqual(
        json.loads('{\n  "enumValue": 1,\n  "repeatedEnumValue": [0, 1]\n}\n'),
        json.loads(
            json_format.MessageToJson(message, use_integers_for_enums=True)
        ),
    )

  def testParseDict(self):
    expected = 12345
    js_dict = {'int32Value': expected}
    message = json_format_proto3_pb2.TestMessage()
    json_format.ParseDict(js_dict, message)
    self.assertEqual(expected, message.int32_value)

  def testParseDictAcceptsPairValueTuples(self):
    expected = [1, 2, 3]
    js_dict = {'repeatedInt32Value': (1, 2, 3)}
    message = json_format_proto3_pb2.TestMessage()
    json_format.ParseDict(js_dict, message)
    self.assertEqual(expected, message.repeated_int32_value)

  def testParseDictAcceptsRepeatedValueTuples(self):
    expected = json_format_proto3_pb2.TestListValue(
        repeated_value=[
            struct_pb2.ListValue(
                values=[
                    struct_pb2.Value(number_value=4),
                    struct_pb2.Value(number_value=5),
                ]
            ),
            struct_pb2.ListValue(values=[struct_pb2.Value(number_value=6)]),
        ]
    )
    js_dict = {'repeated_value': ((4, 5), (6,))}
    message = json_format_proto3_pb2.TestListValue()
    json_format.ParseDict(js_dict, message)
    self.assertEqual(expected, message)

  def testParseDictAnyDescriptorPoolMissingType(self):
    # Confirm that ParseDict does not raise ParseError with default pool
    js_dict = {
        'any_value': {
            '@type': 'type.googleapis.com/proto3.MessageType',
            'value': 1234,
        }
    }
    json_format.ParseDict(js_dict, any_test_pb2.TestAny())
    # Check ParseDict raises ParseError with empty pool
    js_dict = {
        'any_value': {
            '@type': 'type.googleapis.com/proto3.MessageType',
            'value': 1234,
        }
    }
    with self.assertRaises(json_format.ParseError) as cm:
      empty_pool = descriptor_pool.DescriptorPool()
      json_format.ParseDict(
          js_dict, any_test_pb2.TestAny(), descriptor_pool=empty_pool
      )
    self.assertEqual(
        str(cm.exception),
        'Failed to parse any_value field: Can not find message descriptor by'
        ' type_url: type.googleapis.com/proto3.MessageType at '
        'TestAny.any_value.',
    )

  def testParseDictNestedAnyDescriptorPoolMissingType(self):
    # Confirm that ParseDict nondestructive with empty pool
    js_dict = {
        '@type': 'type.googleapis.com/google.protobuf.Any',
        'value': {
            '@type': 'type.googleapis.com/proto2_unittest.TestAny',
            'any_value': {
                '@type': 'type.googleapis.com/UnknownMessageType',
            },
        },
    }
    js_dict_copy = json.loads(json.dumps(js_dict))
    with self.assertRaises(json_format.ParseError) as cm:
      json_format.ParseDict(js_dict_copy, any_pb2.Any())
    self.assertEqual(
        str(cm.exception),
        'Failed to parse any_value field: Can not find message descriptor by'
        ' type_url: type.googleapis.com/UnknownMessageType at'
        ' Any.value.any_value.',
    )
    self.assertEqual(js_dict, js_dict_copy)

  def testParseDictUnknownValueType(self):
    class UnknownClass(object):

      def __repr__(self):
        return 'v'

    message = json_format_proto3_pb2.TestValue()
    self.assertRaisesRegex(
        json_format.ParseError,
        r"Value v has unexpected type <class '.*\.UnknownClass'>.",
        json_format.ParseDict,
        {'value': UnknownClass()},
        message,
    )

  def testMessageToDict(self):
    message = json_format_proto3_pb2.TestMessage()
    message.int32_value = 12345
    expected = {'int32Value': 12345}
    self.assertEqual(expected, json_format.MessageToDict(message))

  def testJsonName(self):
    message = json_format_proto3_pb2.TestCustomJsonName()
    message.value = 12345
    self.assertEqual(
        '{\n  "@value": 12345\n}', json_format.MessageToJson(message)
    )
    parsed_message = json_format_proto3_pb2.TestCustomJsonName()
    self.CheckParseBack(message, parsed_message)

  def testSortKeys(self):
    # Testing sort_keys is not perfectly working, as by random luck we could
    # get the output sorted. We just use a selection of names.
    message = json_format_proto3_pb2.TestMessage(
        bool_value=True,
        int32_value=1,
        int64_value=3,
        uint32_value=4,
        string_value='bla',
    )
    self.assertEqual(
        json_format.MessageToJson(message, sort_keys=True),
        # We use json.dumps() instead of a hardcoded string due to differences
        # between Python 2 and Python 3.
        json.dumps(
            {
                'boolValue': True,
                'int32Value': 1,
                'int64Value': '3',
                'uint32Value': 4,
                'stringValue': 'bla',
            },
            indent=2,
            sort_keys=True,
        ),
    )

  def testNestedRecursiveLimit(self):
    message = unittest_pb2.NestedTestAllTypes()
    self.assertRaisesRegex(
        json_format.ParseError,
        'Message too deep. Max recursion depth is 3',
        json_format.Parse,
        '{"child": {"child": {"child" : {}}}}',
        message,
        max_recursion_depth=3,
    )
    # The following one can pass
    json_format.Parse(
        '{"payload": {}, "child": {"child":{}}}', message, max_recursion_depth=3
    )

  def testJsonNameConflictSerilize(self):
    message = more_messages_pb2.ConflictJsonName(value=2)
    self.assertEqual(
        json.loads('{"old_value": 2}'),
        json.loads(json_format.MessageToJson(message)),
    )

    new_message = more_messages_pb2.ConflictJsonName(new_value=2)
    self.assertEqual(
        json.loads('{"value": 2}'),
        json.loads(json_format.MessageToJson(new_message)),
    )

  def testJsonNameConflictParse(self):
    message = more_messages_pb2.ConflictJsonName()
    json_format.Parse('{"value": 2}', message)
    self.assertEqual(2, message.new_value)
    self.assertEqual(0, message.value)

  def testJsonNameConflictRoundTrip(self):
    message = more_messages_pb2.ConflictJsonName(value=2)
    parsed_message = more_messages_pb2.ConflictJsonName()
    json_string = json_format.MessageToJson(message)
    json_format.Parse(json_string, parsed_message)
    self.assertEqual(message, parsed_message)

    new_message = more_messages_pb2.ConflictJsonName(new_value=2)
    new_parsed_message = more_messages_pb2.ConflictJsonName()
    json_string = json_format.MessageToJson(new_message)
    json_format.Parse(json_string, new_parsed_message)
    self.assertEqual(new_message, new_parsed_message)

  def testOtherParseErrors(self):
    self.CheckError(
        '9',
        "Failed to parse JSON: TypeError: 'int' object is not iterable.",
    )

  def testManyRecursionsRaisesParseError(self):
    num_recursions = 1050
    text = ('{"a":' * num_recursions) + '""' + ('}' * num_recursions)
    with self.assertRaises(json_format.ParseError):
      json_format.Parse(text, json_format_proto3_pb2.TestMessage())


if __name__ == '__main__':
  unittest.main()



"""Utilities for working with FieldMasks."""

from typing import TypeVar

from google.protobuf import message

from google.protobuf import field_mask_pb2
from google.protobuf.util.python import _field_mask_util as field_mask_util_clif


_T = TypeVar("_T", bound=message.Message)


class FieldMaskUtil:
  """Utilities for working with FieldMasks.

  This class mirrors the C++ API structure that uses static methods
  and has a nested MergeOptions class.
  """

  MergeOptions = field_mask_util_clif.FieldMaskUtil.MergeOptions  # pylint: disable=invalid-name

  @classmethod
  def MergeMessageTo(
      cls,
      source: _T,
      mask: field_mask_pb2.FieldMask,
      options: MergeOptions,
      destination: _T,
  ) -> _T:
    """Merges fields specified by a FieldMask from a source message to a copy of a destination message and returns the merged message.

    This function wraps the C++ FieldMaskUtil::MergeMessageTo implementation.
    It merges fields from the source message into a copy of the destination
    message according to the provided mask and options. Neither the source nor
    the destination message object passed to this function is modified.

    Note that this behavior is different from the C++ implementation, which
    modifies the destination message in place.

    Args:
      source: The source protocol buffer message.
      mask: A FieldMask specifying which fields to merge.
      options: An instance of MergeOptions controlling the merge behavior (e.g.,
        replace_message_fields, replace_repeated_fields).
      destination: The destination protocol buffer message.

    Returns:
      The merged message.
    """
    # Call the CLIF-wrapped C++ function
    # It returns a PyObject* which is converted to 'object' in CLIF,
    # and specifically PyBytes in the C++ implementation, so we expect 'bytes'.
    merged_bytes = field_mask_util_clif.MergeMessageToBytes(
        source=source, mask=mask, options=options, destination=destination  # pytype: disable=wrong-arg-types
    )
    return destination.__class__().FromString(merged_bytes)



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests for google.protobuf.proto_builder."""

import collections
import unittest

from google.protobuf import descriptor
from google.protobuf import descriptor_pb2
from google.protobuf import descriptor_pool
from google.protobuf import proto_builder
from google.protobuf import text_format
from google.protobuf.internal import api_implementation


class ProtoBuilderTest(unittest.TestCase):

  def setUp(self):
    super().setUp()

    self.ordered_fields = collections.OrderedDict([
        ('foo', descriptor_pb2.FieldDescriptorProto.TYPE_INT64),
        ('bar', descriptor_pb2.FieldDescriptorProto.TYPE_STRING),
        ])
    self._fields = dict(self.ordered_fields)

  def testMakeSimpleProtoClass(self):
    """Test that we can create a proto class."""
    proto_cls = proto_builder.MakeSimpleProtoClass(
        self._fields,
        full_name='net.proto2.python.public.proto_builder_test.Test')
    proto = proto_cls()
    proto.foo = 12345
    proto.bar = 'asdf'
    self.assertMultiLineEqual(
        'bar: "asdf"\nfoo: 12345\n', text_format.MessageToString(proto))

  def testOrderedFields(self):
    """Test that the field order is maintained when given an OrderedDict."""
    proto_cls = proto_builder.MakeSimpleProtoClass(
        self.ordered_fields,
        full_name='net.proto2.python.public.proto_builder_test.OrderedTest')
    proto = proto_cls()
    proto.foo = 12345
    proto.bar = 'asdf'
    self.assertMultiLineEqual(
        'foo: 12345\nbar: "asdf"\n', text_format.MessageToString(proto))

  def testMakeSameProtoClassTwice(self):
    """Test that the DescriptorPool is used."""
    pool = descriptor_pool.DescriptorPool()
    proto_cls1 = proto_builder.MakeSimpleProtoClass(
        self._fields,
        full_name='net.proto2.python.public.proto_builder_test.Test',
        pool=pool)
    proto_cls2 = proto_builder.MakeSimpleProtoClass(
        self._fields,
        full_name='net.proto2.python.public.proto_builder_test.Test',
        pool=pool)
    self.assertIs(proto_cls1.DESCRIPTOR, proto_cls2.DESCRIPTOR)

  def testMakeLargeProtoClass(self):
    """Test that large created protos don't use reserved field numbers."""
    num_fields = 65000
    fields = {
        'foo%d' % i: descriptor_pb2.FieldDescriptorProto.TYPE_INT64
        for i in range(num_fields)
    }
    if api_implementation.Type() == 'upb':
      with self.assertRaisesRegex(
          TypeError, "Couldn't build proto file into descriptor pool"
      ):
        proto_cls = proto_builder.MakeSimpleProtoClass(
            fields,
            full_name=(
                'net.proto2.python.public.proto_builder_test.LargeProtoTest'
            ),
        )
      return

    proto_cls = proto_builder.MakeSimpleProtoClass(
        fields,
        full_name='net.proto2.python.public.proto_builder_test.LargeProtoTest',
    )

    reserved_field_numbers = set(
        range(
            descriptor.FieldDescriptor.FIRST_RESERVED_FIELD_NUMBER,
            descriptor.FieldDescriptor.LAST_RESERVED_FIELD_NUMBER + 1,
        )
    )
    proto_field_numbers = set(proto_cls.DESCRIPTOR.fields_by_number)
    self.assertFalse(reserved_field_numbers.intersection(proto_field_numbers))


if __name__ == '__main__':
  unittest.main()



import hypothesis as hp
from hypothesis import strategies as st

from google.protobuf import field_mask_pb2
from google.protobuf import struct_pb2
from google.protobuf import timestamp_pb2
from google3.testing.pybase import unittest
from absl.testing import parameterized
from google.protobuf.util.python import field_mask_util
from google.protobuf.util.python.testdata import test_messages_pb2


_ORIGINAL_VALUE = 42
_DEFAULT_MERGE_OPTIONS = field_mask_util.FieldMaskUtil.MergeOptions()


@st.composite
def timestamp_strategy(draw):
  return timestamp_pb2.Timestamp(
      seconds=draw(st.integers(0, 100000)), nanos=draw(st.integers(0, 100000))
  )


@st.composite
def field_mask_strategy(draw):
  return field_mask_pb2.FieldMask(
      paths=draw(st.lists(st.sampled_from(["seconds", "nanos"])))
  )


class FieldMaskUtilTest(parameterized.TestCase):

  def test_merge_message_to_simple(self):
    source = timestamp_pb2.Timestamp(seconds=1, nanos=2)
    mask = field_mask_pb2.FieldMask(paths=["seconds"])
    destination = timestamp_pb2.Timestamp(seconds=3, nanos=4)

    result = field_mask_util.FieldMaskUtil.MergeMessageTo(
        source, mask, _DEFAULT_MERGE_OPTIONS, destination
    )

    self.assertEqual(result.seconds, 1)

  @hp.given(
      source=timestamp_strategy(),
      mask=field_mask_strategy(),
  )
  def test_merge_message_to_hp(
      self, source: timestamp_pb2.Timestamp, mask: field_mask_pb2.FieldMask
  ):
    destination = timestamp_pb2.Timestamp(
        seconds=_ORIGINAL_VALUE, nanos=_ORIGINAL_VALUE
    )

    result = field_mask_util.FieldMaskUtil.MergeMessageTo(
        source, mask, _DEFAULT_MERGE_OPTIONS, destination
    )

    if "seconds" in mask.paths:
      self.assertEqual(result.seconds, source.seconds)
    else:
      self.assertEqual(result.seconds, _ORIGINAL_VALUE)
    if "nanos" in mask.paths:
      self.assertEqual(result.nanos, source.nanos)
    else:
      self.assertEqual(result.nanos, _ORIGINAL_VALUE)

  @parameterized.named_parameters(
      dict(
          testcase_name="replace_repeated_fields",
          replace_repeated_fields=True,
          expected_values=[struct_pb2.Value(string_value="hello")],
      ),
      dict(
          testcase_name="not_replace_repeated_fields",
          replace_repeated_fields=False,
          expected_values=[
              struct_pb2.Value(string_value="world"),
              struct_pb2.Value(string_value="hello"),
          ],
      ),
  )
  def test_merge_message_to_repeated_fields(
      self,
      replace_repeated_fields,
      expected_values,
  ):
    source = struct_pb2.ListValue(
        values=[struct_pb2.Value(string_value="hello")]
    )
    mask = field_mask_pb2.FieldMask(paths=["values"])
    options = field_mask_util.FieldMaskUtil.MergeOptions()
    options.replace_repeated_fields = replace_repeated_fields
    destination = struct_pb2.ListValue(
        values=[struct_pb2.Value(string_value="world")]
    )

    result = field_mask_util.FieldMaskUtil.MergeMessageTo(
        source, mask, options, destination
    )

    self.assertListEqual(list(result.values), expected_values)

  @parameterized.named_parameters(
      dict(
          testcase_name="replace_message_fields",
          replace_message_fields=True,
          expected_nanos=0,
      ),
      dict(
          testcase_name="not_replace_message_fields",
          replace_message_fields=False,
          expected_nanos=4,
      ),
  )
  def test_merge_message_to_message_fields(
      self,
      replace_message_fields,
      expected_nanos,
  ):
    source = test_messages_pb2.TimestampWrapper(
        timestamp=timestamp_pb2.Timestamp(seconds=1)
    )
    mask = field_mask_pb2.FieldMask(paths=["timestamp"])
    options = field_mask_util.FieldMaskUtil.MergeOptions()
    options.replace_message_fields = replace_message_fields
    destination = test_messages_pb2.TimestampWrapper(
        timestamp=timestamp_pb2.Timestamp(seconds=3, nanos=4)
    )

    result = field_mask_util.FieldMaskUtil.MergeMessageTo(
        source, mask, options, destination
    )

    self.assertEqual(result.timestamp.seconds, 1)
    self.assertEqual(result.timestamp.nanos, expected_nanos)

  def test_nested_message_fields(self):
    source = test_messages_pb2.TimestampWrapper(
        timestamp=timestamp_pb2.Timestamp(seconds=1, nanos=2)
    )
    mask = field_mask_pb2.FieldMask(paths=["timestamp.seconds"])
    destination = test_messages_pb2.TimestampWrapper(
        timestamp=timestamp_pb2.Timestamp(seconds=3, nanos=4)
    )
    result = field_mask_util.FieldMaskUtil.MergeMessageTo(
        source, mask, _DEFAULT_MERGE_OPTIONS, destination
    )
    self.assertEqual(result.timestamp.seconds, 1)
    self.assertEqual(result.timestamp.nanos, 4)

  # This is not compliant with https://google.aip.dev/161#map-fields
  # The Python implementation only intends to wrap the C++ one that works this
  # way.
  def test_map_keys_are_unsupported(self):
    source = test_messages_pb2.MapWrapper(
        map={"key1": "new_value", "key2": "new_value"}
    )
    mask = field_mask_pb2.FieldMask(paths=["map.key2"])
    destination = test_messages_pb2.MapWrapper(map={"key1": "original_value"})
    result = field_mask_util.FieldMaskUtil.MergeMessageTo(
        source, mask, _DEFAULT_MERGE_OPTIONS, destination
    )
    self.assertDictEqual(dict(result.map), {"key1": "original_value"})


if __name__ == "__main__":
  unittest.main()



# -*- coding: utf-8 -*-
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests Nextgen Pythonic protobuf APIs."""

import unittest

from google.protobuf import proto_json
from google.protobuf.util import json_format_proto3_pb2


class ProtoJsonTest(unittest.TestCase):

  def test_simple_serialize(self):
    message = json_format_proto3_pb2.TestMessage()
    message.int32_value = 12345
    expected = {'int32Value': 12345}
    self.assertEqual(expected, proto_json.serialize(message))

  def test_simple_parse(self):
    expected = 12345
    js_dict = {'int32Value': expected}
    message = proto_json.parse(json_format_proto3_pb2.TestMessage,
                               js_dict)
    self.assertEqual(expected, message.int32_value)  # pytype: disable=attribute-error


if __name__ == "__main__":
  unittest.main()



# -*- coding: utf-8 -*-
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Unittest for reflection.py, with C++ protos linked in."""

import copy
import unittest

from google.protobuf.internal import testing_refleaks
from absl.testing import parameterized
from google.protobuf import unittest_pb2
from google.protobuf import unittest_proto3_arena_pb2


@parameterized.named_parameters(
    ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
)
@testing_refleaks.TestCase
class ReflectionTest(unittest.TestCase):

  def testEmptyCompositeContainerDeepCopy(self, message_module):
    proto1 = message_module.NestedTestAllTypes(
        payload=message_module.TestAllTypes(optional_string='foo')
    )
    nested2 = copy.deepcopy(proto1.child.repeated_child)
    self.assertEqual(0, len(nested2))


if __name__ == '__main__':
  unittest.main()



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Test use of numpy types with repeated and non-repeated scalar fields."""

from datetime import datetime
import unittest

from google.protobuf.internal import api_implementation
from google.protobuf.internal import testing_refleaks
import numpy as np

from absl.testing import parameterized
from google.protobuf import unittest_pb2
from google.protobuf import unittest_proto3_arena_pb2
from google.protobuf.util import json_format_pb2

message = unittest_pb2.TestAllTypes()
np_float_scalar = np.float64(0.0)
np_1_float_array = np.zeros(shape=(1,), dtype=np.float64)
np_2_float_array = np.zeros(shape=(2,), dtype=np.float64)
np_11_float_array = np.zeros(shape=(1, 1), dtype=np.float64)
np_22_float_array = np.zeros(shape=(2, 2), dtype=np.float64)

np_int_scalar = np.int64(0)
np_1_int_array = np.zeros(shape=(1,), dtype=np.int64)
np_2_int_array = np.zeros(shape=(2,), dtype=np.int64)
np_11_int_array = np.zeros(shape=(1, 1), dtype=np.int64)
np_22_int_array = np.zeros(shape=(2, 2), dtype=np.int64)

np_uint_scalar = np.uint64(0)
np_1_uint_array = np.zeros(shape=(1,), dtype=np.uint64)
np_2_uint_array = np.zeros(shape=(2,), dtype=np.uint64)
np_11_uint_array = np.zeros(shape=(1, 1), dtype=np.uint64)
np_22_uint_array = np.zeros(shape=(2, 2), dtype=np.uint64)

np_bool_scalar = np.bool_(False)
np_1_bool_array = np.zeros(shape=(1,), dtype=np.bool_)
np_2_bool_array = np.zeros(shape=(2,), dtype=np.bool_)
np_11_bool_array = np.zeros(shape=(1, 1), dtype=np.bool_)
np_22_bool_array = np.zeros(shape=(2, 2), dtype=np.bool_)


@testing_refleaks.TestCase
class NumpyIntProtoTest(unittest.TestCase):

  # Assigning dim 1 ndarray of ints to repeated field should pass
  def testNumpyDim1IntArrayToRepeated_IsValid(self):
    message.repeated_int64[:] = np_1_int_array
    message.repeated_int64[:] = np_2_int_array

    message.repeated_uint64[:] = np_1_uint_array
    message.repeated_uint64[:] = np_2_uint_array

  # Assigning dim 2 ndarray of ints to repeated field should fail
  def testNumpyDim2IntArrayToRepeated_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.repeated_int64[:] = np_11_int_array
    with self.assertRaises(TypeError):
      message.repeated_int64[:] = np_22_int_array

    with self.assertRaises(TypeError):
      message.repeated_uint64[:] = np_11_uint_array
    with self.assertRaises(TypeError):
      message.repeated_uint64[:] = np_22_uint_array

  # Assigning any ndarray of floats to repeated int field should fail
  def testNumpyFloatArrayToRepeated_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.repeated_int64[:] = np_1_float_array
    with self.assertRaises(TypeError):
      message.repeated_int64[:] = np_11_float_array
    with self.assertRaises(TypeError):
      message.repeated_int64[:] = np_22_float_array

  # Assigning any np int to scalar field should pass
  def testNumpyIntScalarToScalar_IsValid(self):
    message.optional_int64 = np_int_scalar
    message.optional_uint64 = np_uint_scalar

  # Assigning any ndarray of ints to scalar field should fail
  def testNumpyIntArrayToScalar_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.optional_int64 = np_1_int_array
    with self.assertRaises(TypeError):
      message.optional_int64 = np_11_int_array
    with self.assertRaises(TypeError):
      message.optional_int64 = np_22_int_array

    with self.assertRaises(TypeError):
      message.optional_uint64 = np_1_uint_array
    with self.assertRaises(TypeError):
      message.optional_uint64 = np_11_uint_array
    with self.assertRaises(TypeError):
      message.optional_uint64 = np_22_uint_array

  # Assigning any ndarray of floats to scalar field should fail
  def testNumpyFloatArrayToScalar_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.optional_int64 = np_1_float_array
    with self.assertRaises(TypeError):
      message.optional_int64 = np_11_float_array
    with self.assertRaises(TypeError):
      message.optional_int64 = np_22_float_array


@testing_refleaks.TestCase
class NumpyFloatProtoTest(unittest.TestCase):

  # Assigning dim 1 ndarray of floats to repeated field should pass
  def testNumpyDim1FloatArrayToRepeated_IsValid(self):
    message.repeated_float[:] = np_1_float_array
    message.repeated_float[:] = np_2_float_array

  # Assigning dim 2 ndarray of floats to repeated field should fail
  def testNumpyDim2FloatArrayToRepeated_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.repeated_float[:] = np_11_float_array
    with self.assertRaises(TypeError):
      message.repeated_float[:] = np_22_float_array

  # Assigning any np float to scalar field should pass
  def testNumpyFloatScalarToScalar_IsValid(self):
    message.optional_float = np_float_scalar

  # Assigning any ndarray of float to scalar field should fail
  def testNumpyFloatArrayToScalar_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.optional_float = np_1_float_array
    with self.assertRaises(TypeError):
      message.optional_float = np_11_float_array
    with self.assertRaises(TypeError):
      message.optional_float = np_22_float_array


@testing_refleaks.TestCase
class NumpyBoolProtoTest(unittest.TestCase):

  # Assigning dim 1 ndarray of bool to repeated field should pass
  def testNumpyDim1BoolArrayToRepeated_IsValid(self):
    message.repeated_bool[:] = np_1_bool_array
    message.repeated_bool[:] = np_2_bool_array

  # Assigning dim 2 ndarray of bool to repeated field should fail
  def testNumpyDim2BoolArrayToRepeated_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.repeated_bool[:] = np_11_bool_array
    with self.assertRaises(TypeError):
      message.repeated_bool[:] = np_22_bool_array

  # Assigning any np bool to scalar field should pass
  def testNumpyBoolScalarToScalar_IsValid(self):
    message.optional_bool = np_bool_scalar

  # Assigning any ndarray of bool to scalar field should fail
  def testNumpyBoolArrayToScalar_RaisesTypeError(self):
    with self.assertRaises(TypeError):
      message.optional_bool = np_1_bool_array
    with self.assertRaises(TypeError):
      message.optional_bool = np_11_bool_array
    with self.assertRaises(TypeError):
      message.optional_bool = np_22_bool_array


@testing_refleaks.TestCase
class NumpyProtoIndexingTest(unittest.TestCase):

  def testNumpyIntScalarIndexing_Passes(self):
    data = unittest_pb2.TestAllTypes(repeated_int64=[0, 1, 2])
    self.assertEqual(0, data.repeated_int64[np.int64(0)])

  def testNumpyNegative1IntScalarIndexing_Passes(self):
    data = unittest_pb2.TestAllTypes(repeated_int64=[0, 1, 2])
    self.assertEqual(2, data.repeated_int64[np.int64(-1)])

  def testNumpyFloatScalarIndexing_Fails(self):
    data = unittest_pb2.TestAllTypes(repeated_int64=[0, 1, 2])
    with self.assertRaises(TypeError):
      _ = data.repeated_int64[np.float64(0.0)]

  def testNumpyIntArrayIndexing_Fails(self):
    data = unittest_pb2.TestAllTypes(repeated_int64=[0, 1, 2])
    with self.assertRaises(TypeError):
      _ = data.repeated_int64[np.array([0])]
    with self.assertRaises(TypeError):
      _ = data.repeated_int64[np.ndarray((1,), buffer=np.array([0]), dtype=int)]
    with self.assertRaises(TypeError):
      _ = data.repeated_int64[np.ndarray((1, 1),
                                         buffer=np.array([0]),
                                         dtype=int)]


@testing_refleaks.TestCase
class NumpyBindingTest(parameterized.TestCase):

  @parameterized.product(
      message_module=[unittest_pb2, unittest_proto3_arena_pb2],
      field_name=[
          'repeated_int32',
          'repeated_int64',
          'repeated_uint32',
          'repeated_uint64',
          'repeated_sint32',
          'repeated_sint64',
          'repeated_fixed32',
          'repeated_fixed64',
          'repeated_sfixed32',
          'repeated_sfixed64',
          'repeated_float',
          'repeated_double',
      ],
  )
  def test_simple_np_array_from_repeated(self, message_module, field_name):
    m = message_module.TestAllTypes()
    field = getattr(m, field_name)
    field.append(42)
    field.append(127)
    arr = np.asarray(field)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_equal(arr, np.array([42, 127]))

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_simple_np_array_from_repeated_continue(self, message_module):
    m = message_module.TestAllTypes()
    m.repeated_nested_enum.extend([1, 2, 3])
    arr = np.asarray(m.repeated_nested_enum)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_equal(arr, np.array([1, 2, 3]))

    m.repeated_bool.append(False)
    m.repeated_bool.append(True)
    arr = np.asarray(m.repeated_bool)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_equal(arr, np.array([False, True]))

    m.repeated_string.extend([
        'One',
        'Two',
        'Three',
    ])
    arr = np.asarray(m.repeated_string)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_equal(arr, np.array(['One', 'Two', 'Three']))

    m.repeated_bytes.append(b'1')
    m.repeated_bytes.append(b'2')
    m.repeated_bytes.append(b'3')
    arr = np.asarray(m.repeated_bytes)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_equal(arr, np.array([b'1', b'2', b'3']))

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_simple_n_array_from_repeated_message(self, message_module):
    m = message_module.TestAllTypes(
        repeated_nested_message=(
            message_module.TestAllTypes.NestedMessage(bb=9),
            message_module.TestAllTypes.NestedMessage(bb=8),
        )
    )
    arr = np.array(m.repeated_nested_message)
    self.assertIsInstance(arr, np.ndarray)
    self.assertEqual(arr[0].bb, 9)
    self.assertEqual(arr[1].bb, 8)

  @parameterized.product(
      message_module=[unittest_pb2, unittest_proto3_arena_pb2],
      field_name=[
          'repeated_int32',
          'repeated_int64',
          'repeated_sint32',
          'repeated_sint64',
          'repeated_sfixed32',
          'repeated_sfixed64',
          'repeated_float',
          'repeated_double',
      ],
  )
  def test_numpy_signed_arrays_from_repeated(self, message_module, field_name):
    m = message_module.TestAllTypes()
    field = getattr(m, field_name)
    field.append(-42)
    field.append(0)
    field.append(127)
    arr = np.asarray(field)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_equal(arr, np.array([-42, 0, 127]))

  @parameterized.product(
      message_module=[unittest_pb2, unittest_proto3_arena_pb2],
      field_name=[
          'repeated_int32',
          'repeated_int64',
          'repeated_uint32',
          'repeated_uint64',
          'repeated_sint32',
          'repeated_sint64',
          'repeated_fixed32',
          'repeated_fixed64',
          'repeated_sfixed32',
          'repeated_sfixed64',
          'repeated_float',
          'repeated_double',
          'repeated_nested_enum',
      ],
  )
  def test_numpy_empty_repeated(self, message_module, field_name):
    m = message_module.TestAllTypes()
    field = getattr(m, field_name)
    arr = np.array(field)
    arr2 = np.array(field, dtype=np.int8)
    self.assertIsInstance(arr, np.ndarray)
    self.assertIsInstance(arr2, np.ndarray)
    np.testing.assert_equal(arr, np.array([]))
    np.testing.assert_equal(arr2, np.array([], dtype=np.int8))

  @parameterized.product(
      message_module=[unittest_pb2, unittest_proto3_arena_pb2],
      field_name=['packed_sint32', 'packed_sint64'],
  )
  def test_numpy_signed_packed_arrays_from_repeated(
      self, message_module, field_name
  ):
    m = message_module.TestPackedTypes()
    field = getattr(m, field_name)
    field.append(-42)
    field.append(0)
    field.append(127)
    arr = np.asarray(field)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_equal(arr, np.array([-42, 0, 127]))

  @parameterized.product(
      message_module=[unittest_pb2, unittest_proto3_arena_pb2],
      dtype=[
          'int8',
          'int16',
          'int32',
          'int64',
          'float16',
          'float32',
          'float64',
          'str',
          'bool',
          'object',
      ],
  )
  def test_repeated_bytes_to_all_types(self, message_module, dtype):
    m = message_module.TestAllTypes()
    m.repeated_bytes.extend([b'11', b'12'])
    arr = np.asarray(m.repeated_bytes, dtype=dtype)
    self.assertIsInstance(arr, np.ndarray)
    self.assertTrue(arr.flags.contiguous)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_repeated_string_tobytes(self, message_module):
    m = message_module.TestAllTypes(repeated_string=['12'])
    arr = np.array(m.repeated_string)
    self.assertEqual(arr.tobytes(), b'1\x00\x00\x002\x00\x00\x00')

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_repeated_bytes_tobytes(self, message_module):
    m = message_module.TestAllTypes(repeated_bytes=[b'11', b'12', b'13'])
    arr = np.array(m.repeated_bytes)
    np.testing.assert_array_equal(
        arr, np.asarray([b'11', b'12', b'13'], dtype=bytes)
    )
    self.assertEqual(arr.tobytes(), b'111213')

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_repeated_string_none_dtype(self, message_module):
    m = message_module.TestAllTypes()
    m.repeated_string.extend(['12', '2321'])
    arr = np.asarray(m.repeated_string, dtype=None)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_array_equal(arr, np.asarray(['12', '2321'], dtype=str))

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_repeated_string_int8_dtype(self, message_module):
    m = message_module.TestAllTypes()
    m.repeated_string.extend(['123', '-15'])
    arr = np.asarray(m.repeated_string, dtype=np.int8)
    self.assertIsInstance(arr, np.ndarray)
    self.assertEqual(arr.dtype, np.int8)
    np.testing.assert_array_equal(arr, np.asarray([123, -15], dtype=np.int8))

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_repeated_bytes_none_dtype(self, message_module):
    m = message_module.TestAllTypes()
    m.repeated_bytes.append(bytes([122, 124]))
    m.repeated_bytes.append(bytes([13]))
    arr = np.asarray(m.repeated_bytes, dtype=None)
    self.assertIsInstance(arr, np.ndarray)
    expected = np.asarray([b'\x7A\x7C', b'\x0d'])
    np.testing.assert_array_equal(arr, expected)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_repeated_bytes_object_dtype(self, message_module):
    m = message_module.TestAllTypes()
    t = np.array([b'932', b'124\x00'], dtype=object)
    m.repeated_bytes.extend(t)
    ss = m.SerializeToString()
    m2 = message_module.TestAllTypes.FromString(ss)
    arr = np.asarray(m2.repeated_bytes, dtype=object)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_array_equal(arr, t)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_empty_list_object_dtype(self, message_module):
    m = message_module.TestAllTypes()
    t = np.array([], dtype=object)
    m.repeated_bytes.extend(t)
    ss = m.SerializeToString()
    m2 = message_module.TestAllTypes.FromString(ss)
    arr = np.asarray(m2.repeated_bytes, dtype=object)
    self.assertIsInstance(arr, np.ndarray)
    np.testing.assert_array_equal(arr, t)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_default_dtype(self, message_module):
    m = message_module.TestAllTypes(
        repeated_int32=[1, 2],
        repeated_uint32=[2],
        repeated_int64=[1],
        repeated_uint64=[1],
        repeated_float=[1],
        repeated_double=[0.1],
        repeated_string=['1'],
        repeated_bytes=[b'123'],
        repeated_bool=[True],
    )
    self.assertEqual(np.array(m.repeated_int32).dtype, np.int32)
    self.assertEqual(np.array(m.repeated_uint32).dtype, np.uint32)
    self.assertEqual(np.array(m.repeated_int64).dtype, np.int64)
    self.assertEqual(np.array(m.repeated_uint64).dtype, np.uint64)
    self.assertEqual(np.array(m.repeated_float).dtype, np.float32)
    self.assertEqual(np.array(m.repeated_double).dtype, np.float64)
    self.assertEqual(np.array(m.repeated_string).dtype, np.dtype('<U1'))
    self.assertEqual(np.array(m.repeated_bytes).dtype, np.dtype('S3'))
    self.assertEqual(np.array(m.repeated_bool).dtype, np.dtype('bool'))
    message = json_format_pb2.TestRepeatedEnum(
        repeated_enum=[json_format_pb2.BUFFER]
    )
    self.assertEqual(np.array(message.repeated_enum).dtype, np.int32)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_set_dtype(self, message_module):
    m = message_module.TestAllTypes(
        repeated_int32=[1, 2],
        repeated_uint32=[2],
        repeated_int64=[1],
        repeated_uint64=[1],
        repeated_float=[1.2, 1],
        repeated_double=[0.1],
        repeated_string=['1'],
        repeated_bytes=[b'123'],
        repeated_bool=[True],
    )
    arr = np.array(m.repeated_float)
    self.assertEqual(np.array(m.repeated_int32, dtype=np.int32).dtype, np.int32)
    self.assertEqual(
        np.array(m.repeated_uint32, dtype=np.int32).dtype, np.int32
    )
    self.assertEqual(
        np.array(m.repeated_int64, dtype=np.uint32).dtype, np.uint32
    )
    self.assertEqual(
        np.array(m.repeated_uint64, dtype=np.uint32).dtype, np.uint32
    )
    self.assertEqual(
        np.array(m.repeated_float, dtype=np.float32).dtype, np.float32
    )
    self.assertEqual(
        np.array(m.repeated_double, dtype=np.float32).dtype, np.float32
    )
    self.assertEqual(
        np.array(m.repeated_string, dtype=object).dtype, np.dtype('O')
    )
    self.assertEqual(
        np.array(m.repeated_bytes, dtype=object).dtype, np.dtype('O')
    )
    self.assertEqual(np.array(m.repeated_bool, dtype=np.int32).dtype, np.int32)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_empty_repeated_default_dtype(self, message_module):
    m = message_module.TestAllTypes()
    self.assertEqual(np.array(m.repeated_int32).dtype, np.int32)
    self.assertEqual(np.array(m.repeated_uint32).dtype, np.uint32)
    self.assertEqual(np.array(m.repeated_int64).dtype, np.int64)
    self.assertEqual(np.array(m.repeated_uint64).dtype, np.uint64)
    self.assertEqual(np.array(m.repeated_float).dtype, np.float32)
    self.assertEqual(np.array(m.repeated_double).dtype, np.float64)
    self.assertEqual(np.array(m.repeated_string).dtype, np.dtype('<U1'))
    self.assertEqual(np.array(m.repeated_bytes).dtype, np.dtype('S1'))

  @parameterized.product(
      message_module=[unittest_pb2, unittest_proto3_arena_pb2],
      field_name=[
          'repeated_int32',
          'repeated_int64',
          'repeated_uint32',
          'repeated_uint64',
          'repeated_sint32',
          'repeated_sint64',
          'repeated_fixed32',
          'repeated_fixed64',
          'repeated_sfixed32',
          'repeated_sfixed64',
          'repeated_float',
          'repeated_double',
          'repeated_nested_enum',
      ],
  )
  def test_empty_repeated_set_dtype(self, message_module, field_name):
    m = message_module.TestAllTypes()
    field = getattr(m, field_name)
    self.assertEqual(np.array(field, dtype=np.int32).dtype, np.int32)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_nested_message(self, message_module):
    m = message_module.NestedTestAllTypes()
    arr = np.array(m.child.payload.repeated_float)
    self.assertEqual(arr.dtype, np.float32)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_float_compare(self, message_module):
    m = message_module.TestAllTypes()
    expected = [87.5011, 1.1]
    m.repeated_float.extend(expected)
    np.testing.assert_equal(
        np.array(m.repeated_float), np.array(expected, np.float32)
    )
    m.repeated_double.extend(expected)
    np.testing.assert_equal(np.array(m.repeated_double), np.array(expected))

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_nparray_modify(self, message_module):
    m = message_module.TestAllTypes()
    size = 10
    expected = np.full(size, 123, dtype=np.int32)
    m.repeated_int32.extend(expected)
    arr = np.array(m.repeated_int32, np.int32)
    arr[2] = 111
    self.assertEqual(arr[2], 111)
    self.assertEqual(arr[3], 123)
    self.assertEqual(m.repeated_int32[2], 123)

  @parameterized.named_parameters(
      ('_proto2', unittest_pb2), ('_proto3', unittest_proto3_arena_pb2)
  )
  def test_nparray_order(self, message_module):
    m = message_module.TestAllTypes(repeated_int32=[1, 2, 3])
    arr = np.array(m.repeated_int32, order='F')
    np.testing.assert_equal(arr, np.array([1, 2, 3]))


if __name__ == '__main__':
  unittest.main()



# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests for google.protobuf.symbol_database."""

import unittest

from google.protobuf import descriptor
from google.protobuf import descriptor_pool
from google.protobuf import symbol_database
from google.protobuf import unittest_pb2


class SymbolDatabaseTest(unittest.TestCase):

  def _Database(self):
    if descriptor._USE_C_DESCRIPTORS:
      # The C++ implementation does not allow mixing descriptors from
      # different pools.
      db = symbol_database.SymbolDatabase(pool=descriptor_pool.Default())
    else:
      db = symbol_database.SymbolDatabase()
    # Register representative types from unittest_pb2.
    db.RegisterFileDescriptor(unittest_pb2.DESCRIPTOR)
    db.RegisterMessage(unittest_pb2.TestAllTypes)
    db.RegisterMessage(unittest_pb2.TestAllTypes.NestedMessage)
    db.RegisterMessage(unittest_pb2.TestAllTypes.OptionalGroup)
    db.RegisterMessage(unittest_pb2.TestAllTypes.RepeatedGroup)
    db.RegisterEnumDescriptor(unittest_pb2.ForeignEnum.DESCRIPTOR)
    db.RegisterEnumDescriptor(unittest_pb2.TestAllTypes.NestedEnum.DESCRIPTOR)
    db.RegisterServiceDescriptor(unittest_pb2._TESTSERVICE)
    return db

  def testGetSymbol(self):
    self.assertEqual(
        unittest_pb2.TestAllTypes, self._Database().GetSymbol(
            'proto2_unittest.TestAllTypes'))
    self.assertEqual(
        unittest_pb2.TestAllTypes.NestedMessage, self._Database().GetSymbol(
            'proto2_unittest.TestAllTypes.NestedMessage'))
    self.assertEqual(
        unittest_pb2.TestAllTypes.OptionalGroup, self._Database().GetSymbol(
            'proto2_unittest.TestAllTypes.OptionalGroup'))
    self.assertEqual(
        unittest_pb2.TestAllTypes.RepeatedGroup, self._Database().GetSymbol(
            'proto2_unittest.TestAllTypes.RepeatedGroup'))

  def testEnums(self):
    # Check registration of types in the pool.
    self.assertEqual(
        'proto2_unittest.ForeignEnum',
        self._Database().pool.FindEnumTypeByName(
            'proto2_unittest.ForeignEnum').full_name)
    self.assertEqual(
        'proto2_unittest.TestAllTypes.NestedEnum',
        self._Database().pool.FindEnumTypeByName(
            'proto2_unittest.TestAllTypes.NestedEnum').full_name)

  def testFindMessageTypeByName(self):
    self.assertEqual(
        'proto2_unittest.TestAllTypes',
        self._Database().pool.FindMessageTypeByName(
            'proto2_unittest.TestAllTypes').full_name)
    self.assertEqual(
        'proto2_unittest.TestAllTypes.NestedMessage',
        self._Database().pool.FindMessageTypeByName(
            'proto2_unittest.TestAllTypes.NestedMessage').full_name)

  def testFindServiceByName(self):
    self.assertEqual(
        'proto2_unittest.TestService',
        self._Database().pool.FindServiceByName(
            'proto2_unittest.TestService').full_name)

  def testFindFileContainingSymbol(self):
    # Lookup based on either enum or message.
    self.assertEqual(
        'google/protobuf/unittest.proto',
        self._Database().pool.FindFileContainingSymbol(
            'proto2_unittest.TestAllTypes.NestedEnum').name)
    self.assertEqual(
        'google/protobuf/unittest.proto',
        self._Database().pool.FindFileContainingSymbol(
            'proto2_unittest.TestAllTypes').name)

  def testFindFileByName(self):
    self.assertEqual(
        'google/protobuf/unittest.proto',
        self._Database().pool.FindFileByName(
            'google/protobuf/unittest.proto').name)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests proto Any APIs."""

import unittest

from google.protobuf import any as proto_any

from google.protobuf import any_pb2
from google.protobuf import unittest_pb2


class AnyTest(unittest.TestCase):

  def test_pack_unpack(self):
    all_types = unittest_pb2.TestAllTypes()
    any_msg = proto_any.pack(all_types)
    all_descriptor = all_types.DESCRIPTOR
    self.assertEqual(
        any_msg.type_url, 'type.googleapis.com/%s' % all_descriptor.full_name
    )

    # Any can be successfully unpacked to the correct message type.
    unpacked_message = unittest_pb2.TestAllTypes()
    self.assertTrue(proto_any.unpack(any_msg, unpacked_message))

    proto_any.unpack_as(any_msg, unittest_pb2.TestAllTypes)

    # Any can't be unpacked to an incorrect message type.
    self.assertFalse(
        proto_any.unpack(any_msg, unittest_pb2.TestAllTypes.NestedMessage())
    )

    with self.assertRaises(TypeError) as catcher:
      proto_any.unpack_as(any_msg, unittest_pb2.TestAllTypes.NestedMessage)
    self.assertIn('Attempted to unpack', catcher.exception.args[0])

  def test_type_name(self):
    all_types = unittest_pb2.TestAllTypes()
    any_msg = proto_any.pack(all_types)
    self.assertEqual(
        proto_any.type_name(any_msg), 'proto2_unittest.TestAllTypes'
    )

  def test_is_type(self):
    all_types = unittest_pb2.TestAllTypes()
    any_msg = proto_any.pack(all_types)
    all_descriptor = all_types.DESCRIPTOR
    self.assertTrue(proto_any.is_type(any_msg, all_descriptor))

    empty_any = any_pb2.Any()
    self.assertFalse(proto_any.is_type(empty_any, all_descriptor))


if __name__ == '__main__':
  unittest.main()



"""Validate the YAML files for GitHub Actions workflows.

TODO: b/359303016 - convert to use unittest
"""

import os
import re

import yaml


def check_ctest(namein, jobin, filein, obj):
  """Checks that the ctest command is not in a mode that allows missing tests to pass."""
  for key in obj:
    if (
        isinstance(obj[key], str)
        and 'ctest' in obj[key]
        and 'ctest --no-tests=error' not in obj[key]
    ):
      raise ValueError(
          'Step %s in job %s from file %s runs ctest in a mode that allows'
          ' missing tests to pass.  Add --no-tests=error to the command and'
          ' build with -Dprotobuf_BUILD_TESTS=ON.' % (namein, jobin, filein)
      )
    elif isinstance(obj[key], dict):
      check_ctest(namein, jobin, filein, obj[key])


# Ensure every job is in the list of blocking jobs.
with open(
    os.path.join(os.path.dirname(__file__), '../workflows/test_runner.yml'), 'r'
) as f:
  data = yaml.safe_load(f)

  # List of all YAML files that are used by jobs in the test_runner.yml file.
  yaml_files = []

  # Get a list of all jobs in the test_runner, except for the blocking job and
  # the tag removal job, which is not always run.
  all_jobs = list(data['jobs'].keys())
  all_jobs.remove('all-blocking-tests')
  all_jobs.remove('remove-tag')

  passed = True
  blocking_jobs = data['jobs']['all-blocking-tests']['needs']

  for job in all_jobs:
    if 'uses' in data['jobs'][job]:
      yaml_files.append(
          os.path.join(
              os.path.dirname(__file__),
              '../workflows',
              os.path.basename(data['jobs'][job]['uses']),
          )
      )
    if job not in blocking_jobs:
      passed = False
      raise ValueError('Job %s is not in the list of blocking jobs.' % job)

  print('PASSED: All jobs are in the list of blocking jobs.')

# Ensure every job with a continuous prefix conditions every step on whether we
# are in a continuous run.
for file in yaml_files:
  with open(file, 'r') as f:
    data = yaml.safe_load(f)
    jobs = data['jobs']
    for job in jobs:
      if 'steps' not in jobs[job]:
        continue
      continuous_condition = 'inputs.continuous-prefix' in jobs[job]['name']
      steps = jobs[job]['steps']
      for step in steps:
        if 'name' in step:
          name = step['name']
        elif 'with' in step and 'name' in step['with']:
          name = step['with']['name']
        else:
          raise ValueError(
              'Step in job %s from file %s does not have a name.' % (job, file)
          )
        if continuous_condition and 'continuous-run' not in step.get('if', ''):
          raise ValueError(
              'Step %s in job %s from file %s does not check the continuous-run'
              ' condition' % (name, job, file)
          )
        if not continuous_condition and 'continuous-run' in step.get('if', ''):
          raise ValueError(
              'Step %s in job %s from file %s checks the continuous-run'
              ' condition but the job does not contain the continuous-prefix'
              % (name, job, file)
          )
        check_ctest(name, job, file, step)

print('PASSED: All steps in all jobs check the continuous-run condition.')



#!/usr/bin/env python
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Script to update a failure list file to add/remove failures.

When adding, will attempt to place them in their correct lexicographical
position relative to other test names. This requires that the failure list is
already sorted. This does not guarantee that the tests will appear neatly one
after the other, as there may be comments in between. If the failure list
is not sorted, sorted blocks may be produced, but the list as a whole will not.
Lastly, be wary of lists that have tests stripped from OSS; if you catch that
a test was added inside a stripped block, you may need to move it out.

This is sort of like comm(1), except it recognizes comments and ignores them.
"""

import argparse

parser = argparse.ArgumentParser(
    description='Adds/removes failures from the failure list.')
parser.add_argument('filename', type=str, help='failure list file to update')
parser.add_argument('--add', dest='add_list', action='append')
parser.add_argument('--remove', dest='remove_list', action='append')

DEFAULT_ALIGNMENT = 114
args = parser.parse_args()

add_set = set()
remove_set = set()

# Adds test + failure message
for add_file in (args.add_list or []):
  with open(add_file) as f:
    for line in f:
      add_set.add(line)

# We only remove by name
for remove_file in (args.remove_list or []):
  with open(remove_file) as f:
    for line in f:
      if line in add_set:
        raise Exception("Asked to both add and remove test: " + line)
      remove_set.add(line.split('#')[0].strip())

add_list = sorted(add_set, reverse=True)

with open(args.filename) as in_file:
  existing_list = in_file.read()

with open(args.filename, 'w') as f:
  split_lines = existing_list.splitlines(True)
  # Make sure that we have one newline for potential tests that have to be
  # added at the end.
  split_lines[-1] = split_lines[-1].rstrip() + '\n'
  for line in split_lines:
    test = line.split('#')[0].strip()
    # As long as the tests we are adding appear lexicographically before our
    # read test, put them first followed by our read test.
    while add_list and test > add_list[-1]:
      f.write(add_list.pop())
    if test not in remove_set:
      f.write(line)
  # Any remaining tests are added at the end
  while add_list:
    f.write(add_list.pop())

# Update our read of the existing file
with open(args.filename, 'r') as f:
  existing_list = f.read()

# Actual alignment of failure messages to 'DEFAULT_ALIGNMENT'
# If test name exceeds DEFAULT_ALIGNMENT, it cannot and will not be aligned.
with open(args.filename, 'w') as f:
  for line in existing_list.splitlines(True):
    split = line.split('#', 1)
    test = split[0].strip()
    if len(split) > 1 and test:
      message = split[1].lstrip()
      spaces = ' ' * (DEFAULT_ALIGNMENT - len(test))
      line = test + spaces + ' # ' + message
      f.write(line)
    else:  # ignore blank lines/lines that do not have '#'/comments
      f.write(line)



# -*- coding: utf-8 -*-
# Protocol Buffers - Google's data interchange format
# Copyright 2008 Google Inc.  All rights reserved.
#
# Use of this source code is governed by a BSD-style
# license that can be found in the LICENSE file or at
# https://developers.google.com/open-source/licenses/bsd

"""Tests Nextgen Pythonic protobuf APIs."""

import io
import unittest

from google.protobuf import proto
from google.protobuf.internal import encoder
from google.protobuf.internal import test_proto2_pb2
from google.protobuf.internal import test_util
from google.protobuf.internal import testing_refleaks

from absl.testing import parameterized
from google.protobuf import unittest_pb2
from google.protobuf import unittest_proto3_arena_pb2


@parameterized.named_parameters(
    ('_proto2', unittest_pb2),
    ('_proto3', unittest_proto3_arena_pb2),
)
@testing_refleaks.TestCase
class ProtoTest(unittest.TestCase):

  def test_simple_serialize_parse(self, message_module):
    msg = message_module.TestAllTypes()
    test_util.SetAllFields(msg)
    serialized_data = proto.serialize(msg)
    parsed_msg = proto.parse(message_module.TestAllTypes, serialized_data)
    self.assertEqual(msg, parsed_msg)

  def test_serialize_parse_length_prefixed_empty(self, message_module):
    empty_alltypes = message_module.TestAllTypes()
    out = io.BytesIO()
    proto.serialize_length_prefixed(empty_alltypes, out)

    input_bytes = io.BytesIO(out.getvalue())
    msg = proto.parse_length_prefixed(message_module.TestAllTypes, input_bytes)

    self.assertEqual(msg, empty_alltypes)

  def test_parse_length_prefixed_truncated(self, message_module):
    out = io.BytesIO()
    encoder._VarintEncoder()(out.write, 9999)
    msg = message_module.TestAllTypes(optional_int32=1)
    out.write(proto.serialize(msg))

    input_bytes = io.BytesIO(out.getvalue())
    with self.assertRaises(ValueError) as context:
      proto.parse_length_prefixed(message_module.TestAllTypes, input_bytes)
      self.assertEqual(
          str(context.exception),
          'Truncated message or non-buffered input_bytes: '
          'Expected 9999 bytes but only 2 bytes parsed for '
          'TestAllTypes.',
      )

  def test_serialize_length_prefixed_fake_io(self, message_module):
    class FakeBytesIO(io.BytesIO):

      def write(self, b: bytes) -> int:
        return 0

    msg = message_module.TestAllTypes(optional_int32=123)
    out = FakeBytesIO()
    with self.assertRaises(TypeError) as context:
      proto.serialize_length_prefixed(msg, out)
    self.assertIn(
        'Failed to write complete message (wrote: 0, expected: 2)',
        str(context.exception),
    )

  def test_byte_size(self, message_module):
    msg = message_module.TestAllTypes()
    self.assertEqual(0, proto.byte_size(msg))
    msg.optional_int32 = 123
    self.assertEqual(2, proto.byte_size(msg))

  def test_clear_message(self, message_module):
    msg = message_module.TestAllTypes()
    msg.oneof_uint32 = 11
    msg.repeated_nested_message.add(bb=1)
    proto.clear_message(msg)
    self.assertIsNone(msg.WhichOneof('oneof_field'))
    self.assertEqual(0, len(msg.repeated_nested_message))

  def test_clear_field(self, message_module):
    msg = message_module.TestAllTypes()
    msg.optional_int32 = 123
    self.assertEqual(123, msg.optional_int32)
    proto.clear_field(msg, 'optional_int32')
    self.assertEqual(0, msg.optional_int32)


class SelfFieldTest(unittest.TestCase):

  def test_pytype_allows_unset_self_field(self):
    self.assertEqual(
        test_proto2_pb2.MessageWithSelfField(something=123).something, 123
    )

  def test_pytype_allows_unset_self_and_self_underscore_field(self):
    self.assertEqual(
        test_proto2_pb2.MessageWithSelfAndSelfUnderscoreField(
            something=123
        ).something,
        123,
    )


_EXPECTED_PROTO3 = b'\x04r\x02hi\x06\x08\x01r\x02hi\x06\x08\x02r\x02hi'
_EXPECTED_PROTO2 = b'\x06\x08\x00r\x02hi\x06\x08\x01r\x02hi\x06\x08\x02r\x02hi'


@parameterized.named_parameters(
    ('_proto2', unittest_pb2, _EXPECTED_PROTO2),
    ('_proto3', unittest_proto3_arena_pb2, _EXPECTED_PROTO3),
)
@testing_refleaks.TestCase
class LengthPrefixedWithGolden(unittest.TestCase):

  def test_serialize_length_prefixed(self, message_module, expected):
    number_of_messages = 3

    out = io.BytesIO()
    for index in range(0, number_of_messages):
      msg = message_module.TestAllTypes(
          optional_int32=index, optional_string='hi'
      )
      proto.serialize_length_prefixed(msg, out)

    self.assertEqual(out.getvalue(), expected)

  def test_parse_length_prefixed(self, message_module, input_bytes):
    expected_number_of_messages = 3

    input_io = io.BytesIO(input_bytes)
    index = 0
    while True:
      msg = proto.parse_length_prefixed(message_module.TestAllTypes, input_io)
      if msg is None:
        break
      self.assertEqual(msg.optional_int32, index)
      self.assertEqual(msg.optional_string, 'hi')
      index += 1

    self.assertEqual(index, expected_number_of_messages)


if __name__ == '__main__':
  unittest.main()
